#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function

__metaclass__ = type
DOCUMENTATION = """
module: ios_logging
author: Trishna Guha (@trishnaguha)
short_description: Manage logging on network devices
description:
- This module provides declarative management of logging on Cisco Ios devices.
version_added: 1.0.0
notes:
- Tested against IOS 15.6
options:
  dest:
    description:
    - Destination of the logs.
    - On dest has to be quoted as 'on' or else pyyaml will convert
      to True before it gets to Ansible.
    choices:
    - 'on'
    - host
    - console
    - monitor
    - buffered
    - trap
    type: str
  name:
    description:
    - The hostname or IP address of the destination.
    - Required when I(dest=host).
    type: str
  size:
    description:
    - Size of buffer. The acceptable value is in range from 4096 to 4294967295 bytes.
    type: int
  facility:
    description:
    - Set logging facility.
    type: str
  level:
    description:
    - Set logging severity levels.
    default: debugging
    choices:
    - emergencies
    - alerts
    - critical
    - errors
    - warnings
    - notifications
    - informational
    - debugging
    type: str
  aggregate:
    description: List of logging definitions.
    type: list
    elements: dict
    suboptions:
      dest:
        description:
        - Destination of the logs.
        - On dest has to be quoted as 'on' or else pyyaml will convert
          to True before it gets to Ansible.
        choices:
        - 'on'
        - host
        - console
        - monitor
        - buffered
        - trap
        type: str
      name:
        description:
        - The hostname or IP address of the destination.
        - Required when I(dest=host).
        type: str
      size:
        description:
        - Size of buffer. The acceptable value is in range from 4096 to 4294967295 bytes.
        type: int
      facility:
        description:
        - Set logging facility.
        type: str
      level:
        description:
        - Set logging severity levels.
        type: str
        choices:
        - emergencies
        - alerts
        - critical
        - errors
        - warnings
        - notifications
        - informational
        - debugging
      state:
        description:
        - State of the logging configuration.
        choices:
        - present
        - absent
        type: str
  state:
    description:
    - State of the logging configuration.
    default: present
    choices:
    - present
    - absent
    type: str
extends_documentation_fragment:
- cisco.ios.ios
"""
EXAMPLES = """
- name: configure host logging
  cisco.ios.ios_logging:
    dest: host
    name: 172.16.0.1
    state: present

- name: remove host logging configuration
  cisco.ios.ios_logging:
    dest: host
    name: 172.16.0.1
    state: absent

- name: configure console logging level and facility
  cisco.ios.ios_logging:
    dest: console
    facility: local7
    level: debugging
    state: present

- name: enable logging to all
  cisco.ios.ios_logging:
    dest: on

- name: configure buffer size
  cisco.ios.ios_logging:
    dest: buffered
    size: 5000

- name: Configure logging using aggregate
  cisco.ios.ios_logging:
    aggregate:
    - {dest: console, level: notifications}
    - {dest: buffered, size: 9000}

- name: remove logging using aggregate
  cisco.ios.ios_logging:
    aggregate:
    - {dest: console, level: notifications}
    - {dest: buffered, size: 9000}
    state: absent
"""
RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always
  type: list
  sample:
    - logging facility local7
    - logging host 172.16.0.1
"""
import re
from copy import deepcopy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    remove_default_spec,
    validate_ip_address,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.ios import (
    get_config,
    load_config,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.ios import (
    get_capabilities,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.ios import (
    ios_argument_spec,
)


def validate_size(value, module):
    if value:
        if not int(4096) <= int(value) <= int(4294967295):
            module.fail_json(msg="size must be between 4096 and 4294967295")
        else:
            return value


def map_obj_to_commands(updates, module, os_version):
    dest_group = "console", "monitor", "buffered", "on", "trap"
    commands = list()
    want, have = updates
    for w in want:
        dest = w["dest"]
        name = w["name"]
        size = w["size"]
        facility = w["facility"]
        level = w["level"]
        state = w["state"]
        del w["state"]
        if facility:
            w["dest"] = "facility"
        if state == "absent" and w in have:
            if dest:
                if dest == "host":
                    if "12." in os_version:
                        commands.append("no logging {0}".format(name))
                    else:
                        commands.append("no logging host {0}".format(name))
                elif dest in dest_group:
                    commands.append("no logging {0}".format(dest))
                else:
                    module.fail_json(
                        msg="dest must be among console, monitor, buffered, host, on, trap"
                    )
            if facility:
                commands.append("no logging facility {0}".format(facility))
        if state == "present" and w not in have:
            if facility:
                present = False
                for entry in have:
                    if (
                        entry["dest"] == "facility"
                        and entry["facility"] == facility
                    ):
                        present = True
                if not present:
                    commands.append("logging facility {0}".format(facility))
            if dest == "host":
                if "12." in os_version:
                    commands.append("logging {0}".format(name))
                else:
                    commands.append("logging host {0}".format(name))
            elif dest == "on":
                commands.append("logging on")
            elif dest == "buffered" and size:
                present = False
                for entry in have:
                    if (
                        entry["dest"] == "buffered"
                        and entry["size"] == size
                        and entry["level"] == level
                    ):
                        present = True
                if not present:
                    if level and level != "debugging":
                        commands.append(
                            "logging buffered {0} {1}".format(size, level)
                        )
                    else:
                        commands.append("logging buffered {0}".format(size))
            elif dest:
                dest_cmd = "logging {0}".format(dest)
                if level:
                    dest_cmd += " {0}".format(level)
                commands.append(dest_cmd)
    return commands


def parse_facility(line, dest):
    facility = None
    if dest == "facility":
        match = re.search("logging facility (\\S+)", line, re.M)
        if match:
            facility = match.group(1)
    return facility


def parse_size(line, dest):
    size = None
    if dest == "buffered":
        match = re.search(
            "logging buffered(?: (\\d+))?(?: [a-z]+)?", line, re.M
        )
        if match:
            if match.group(1) is not None:
                size = match.group(1)
            else:
                size = "4096"
    return size


def parse_name(line, dest):
    if dest == "host":
        match = re.search("logging host (\\S+)", line, re.M)
        if match:
            name = match.group(1)
    else:
        name = None
    return name


def parse_level(line, dest):
    level_group = (
        "emergencies",
        "alerts",
        "critical",
        "errors",
        "warnings",
        "notifications",
        "informational",
        "debugging",
    )
    if dest == "host":
        level = "debugging"
    else:
        if dest == "buffered":
            match = re.search(
                "logging buffered(?: \\d+)?(?: ([a-z]+))?", line, re.M
            )
        else:
            match = re.search("logging {0} (\\S+)".format(dest), line, re.M)
        if match and match.group(1) in level_group:
            level = match.group(1)
        else:
            level = "debugging"
    return level


def map_config_to_obj(module):
    obj = []
    dest_group = (
        "console",
        "host",
        "monitor",
        "buffered",
        "on",
        "facility",
        "trap",
    )
    data = get_config(module, flags=["| include logging"])
    for line in data.split("\n"):
        match = re.search("^logging (\\S+)", line, re.M)
        if match:
            if match.group(1) in dest_group:
                dest = match.group(1)
                obj.append(
                    {
                        "dest": dest,
                        "name": parse_name(line, dest),
                        "size": parse_size(line, dest),
                        "facility": parse_facility(line, dest),
                        "level": parse_level(line, dest),
                    }
                )
            elif validate_ip_address(match.group(1)):
                dest = "host"
                obj.append(
                    {
                        "dest": dest,
                        "name": match.group(1),
                        "size": parse_size(line, dest),
                        "facility": parse_facility(line, dest),
                        "level": parse_level(line, dest),
                    }
                )
            else:
                ip_match = re.search(
                    "\\d+\\.\\d+\\.\\d+\\.\\d+", match.group(1), re.M
                )
                if ip_match:
                    dest = "host"
                    obj.append(
                        {
                            "dest": dest,
                            "name": match.group(1),
                            "size": parse_size(line, dest),
                            "facility": parse_facility(line, dest),
                            "level": parse_level(line, dest),
                        }
                    )
    return obj


def map_params_to_obj(module, required_if=None):
    obj = []
    aggregate = module.params.get("aggregate")
    if aggregate:
        for item in aggregate:
            for key in item:
                if item.get(key) is None:
                    item[key] = module.params[key]
            module._check_required_if(required_if, item)
            d = item.copy()
            if d["dest"] != "host":
                d["name"] = None
            if d["dest"] == "buffered":
                if "size" in d:
                    d["size"] = str(validate_size(d["size"], module))
                elif "size" not in d:
                    d["size"] = str(4096)
                else:
                    pass
            if d["dest"] != "buffered":
                d["size"] = None
            obj.append(d)
    else:
        if module.params["dest"] != "host":
            module.params["name"] = None
        if module.params["dest"] == "buffered":
            if not module.params["size"]:
                module.params["size"] = str(4096)
        else:
            module.params["size"] = None
        if module.params["size"] is None:
            obj.append(
                {
                    "dest": module.params["dest"],
                    "name": module.params["name"],
                    "size": module.params["size"],
                    "facility": module.params["facility"],
                    "level": module.params["level"],
                    "state": module.params["state"],
                }
            )
        else:
            obj.append(
                {
                    "dest": module.params["dest"],
                    "name": module.params["name"],
                    "size": str(validate_size(module.params["size"], module)),
                    "facility": module.params["facility"],
                    "level": module.params["level"],
                    "state": module.params["state"],
                }
            )
    return obj


def main():
    """ main entry point for module execution
    """
    element_spec = dict(
        dest=dict(
            type="str",
            choices=["on", "host", "console", "monitor", "buffered", "trap"],
        ),
        name=dict(type="str"),
        size=dict(type="int"),
        facility=dict(type="str"),
        level=dict(
            type="str",
            default="debugging",
            choices=[
                "emergencies",
                "alerts",
                "critical",
                "errors",
                "warnings",
                "notifications",
                "informational",
                "debugging",
            ],
        ),
        state=dict(default="present", choices=["present", "absent"]),
    )
    aggregate_spec = deepcopy(element_spec)
    # remove default in aggregate spec, to handle common arguments
    remove_default_spec(aggregate_spec)
    argument_spec = dict(
        aggregate=dict(type="list", elements="dict", options=aggregate_spec)
    )
    argument_spec.update(element_spec)
    argument_spec.update(ios_argument_spec)
    required_if = [("dest", "host", ["name"])]
    module = AnsibleModule(
        argument_spec=argument_spec,
        required_if=required_if,
        supports_check_mode=True,
    )
    device_info = get_capabilities(module)
    os_version = device_info["device_info"]["network_os_version"]
    warnings = list()
    result = {"changed": False}
    if warnings:
        result["warnings"] = warnings
    want = map_params_to_obj(module, required_if=required_if)
    have = map_config_to_obj(module)
    commands = map_obj_to_commands((want, have), module, os_version)
    result["commands"] = commands
    if commands:
        if not module.check_mode:
            load_config(module, commands)
        result["changed"] = True
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# coding: utf-8 -*-

# (c) 2017, Wayne Witzel III <wayne@riotousliving.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: tower_job_launch
author: "Wayne Witzel III (@wwitzel3)"
short_description: Launch an Ansible Job.
description:
    - Launch an Ansible Tower jobs. See
      U(https://www.ansible.com/tower) for an overview.
options:
    name:
      description:
        - Name of the job template to use.
      required: True
      type: str
      aliases: ['job_template']
    job_type:
      description:
        - Job_type to use for the job, only used if prompt for job_type is set.
      choices: ["run", "check"]
      type: str
    inventory:
      description:
        - Inventory to use for the job, only used if prompt for inventory is set.
      type: str
    credentials:
      description:
        - Credential to use for job, only used if prompt for credential is set.
      type: list
      aliases: ['credential']
      elements: str
    extra_vars:
      description:
        - extra_vars to use for the Job Template.
        - ask_extra_vars needs to be set to True via tower_job_template module
          when creating the Job Template.
      type: dict
    limit:
      description:
        - Limit to use for the I(job_template).
      type: str
    tags:
      description:
        - Specific tags to use for from playbook.
      type: list
      elements: str
    scm_branch:
      description:
        - A specific of the SCM project to run the template on.
        - This is only applicable if your project allows for branch override.
      type: str
    skip_tags:
      description:
        - Specific tags to skip from the playbook.
      type: list
      elements: str
    verbosity:
      description:
        - Verbosity level for this job run
      type: int
      choices: [ 0, 1, 2, 3, 4, 5 ]
    diff_mode:
      description:
        - Show the changes made by Ansible tasks where supported
      type: bool
    credential_passwords:
      description:
        - Passwords for credentials which are set to prompt on launch
      type: dict
extends_documentation_fragment: awx.awx.auth
'''

EXAMPLES = '''
- name: Launch a job
  tower_job_launch:
    job_template: "My Job Template"
  register: job

- name: Launch a job template with extra_vars on remote Tower instance
  tower_job_launch:
    job_template: "My Job Template"
    extra_vars:
      var1: "My First Variable"
      var2: "My Second Variable"
      var3: "My Third Variable"
    job_type: run

- name: Launch a job with inventory and credential
  tower_job_launch:
    job_template: "My Job Template"
    inventory: "My Inventory"
    credential: "My Credential"
  register: job
- name: Wait for job max 120s
  tower_job_wait:
    job_id: "{{ job.id }}"
    timeout: 120
'''

RETURN = '''
id:
    description: job id of the newly launched job
    returned: success
    type: int
    sample: 86
status:
    description: status of newly launched job
    returned: success
    type: str
    sample: pending
'''

from ..module_utils.tower_api import TowerAPIModule


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        name=dict(required=True, aliases=['job_template']),
        job_type=dict(choices=['run', 'check']),
        inventory=dict(default=None),
        # Credentials will be a str instead of a list for backwards compatability
        credentials=dict(type='list', default=None, aliases=['credential'], elements='str'),
        limit=dict(),
        tags=dict(type='list', elements='str'),
        extra_vars=dict(type='dict'),
        scm_branch=dict(),
        skip_tags=dict(type='list', elements='str'),
        verbosity=dict(type='int', choices=[0, 1, 2, 3, 4, 5]),
        diff_mode=dict(type='bool'),
        credential_passwords=dict(type='dict'),
    )

    # Create a module for ourselves
    module = TowerAPIModule(argument_spec=argument_spec)

    optional_args = {}
    # Extract our parameters
    name = module.params.get('name')
    optional_args['job_type'] = module.params.get('job_type')
    inventory = module.params.get('inventory')
    credentials = module.params.get('credentials')
    optional_args['limit'] = module.params.get('limit')
    optional_args['tags'] = module.params.get('tags')
    optional_args['extra_vars'] = module.params.get('extra_vars')
    optional_args['scm_branch'] = module.params.get('scm_branch')
    optional_args['skip_tags'] = module.params.get('skip_tags')
    optional_args['verbosity'] = module.params.get('verbosity')
    optional_args['diff_mode'] = module.params.get('diff_mode')
    optional_args['credential_passwords'] = module.params.get('credential_passwords')

    # Create a datastructure to pass into our job launch
    post_data = {}
    for key in optional_args.keys():
        if optional_args[key]:
            post_data[key] = optional_args[key]

    # Attempt to look up the related items the user specified (these will fail the module if not found)
    if inventory:
        post_data['inventory'] = module.resolve_name_to_id('inventories', inventory)

    if credentials:
        post_data['credentials'] = []
        for credential in credentials:
            post_data['credentials'].append(module.resolve_name_to_id('credentials', credential))

    # Attempt to look up job_template based on the provided name
    job_template = module.get_one('job_templates', **{
        'data': {
            'name': name,
        }
    })

    if job_template is None:
        module.fail_json(msg="Unable to find job template by name {0}".format(name))

    # The API will allow you to submit values to a jb launch that are not prompt on launch.
    # Therefore, we will test to see if anything is set which is not prompt on launch and fail.
    check_vars_to_prompts = {
        'scm_branch': 'ask_scm_branch_on_launch',
        'diff_mode': 'ask_diff_mode_on_launch',
        'extra_vars': 'ask_variables_on_launch',
        'limit': 'ask_limit_on_launch',
        'tags': 'ask_tags_on_launch',
        'skip_tags': 'ask_skip_tags_on_launch',
        'job_type': 'ask_job_type_on_launch',
        'verbosity': 'ask_verbosity_on_launch',
        'inventory': 'ask_inventory_on_launch',
        'credentials': 'ask_credential_on_launch',
    }

    param_errors = []
    for variable_name in check_vars_to_prompts:
        if module.params.get(variable_name) and not job_template[check_vars_to_prompts[variable_name]]:
            param_errors.append("The field {0} was specified but the job template does not allow for it to be overridden".format(variable_name))
    if len(param_errors) > 0:
        module.fail_json(msg="Parameters specified which can not be passed into job template, see errors for details", **{'errors': param_errors})

    # Launch the job
    results = module.post_endpoint(job_template['related']['launch'], **{'data': post_data})

    if results['status_code'] != 201:
        module.fail_json(msg="Failed to launch job, see response for details", **{'response': results})

    module.exit_json(**{
        'changed': True,
        'id': results['json']['id'],
        'status': results['json']['status'],
    })


if __name__ == '__main__':
    main()



from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import pytest

from awx.main.models import Project


@pytest.mark.django_db
def test_create_project(run_module, admin_user, organization, silence_warning):
    result = run_module('tower_project', dict(
        name='foo',
        organization=organization.name,
        scm_type='git',
        scm_url='https://foo.invalid',
        wait=False,
        scm_update_cache_timeout=5
    ), admin_user)
    silence_warning.assert_called_once_with(
        'scm_update_cache_timeout will be ignored since scm_update_on_launch '
        'was not set to true')

    assert result.pop('changed', None), result

    proj = Project.objects.get(name='foo')
    assert proj.scm_url == 'https://foo.invalid'
    assert proj.organization == organization

    result.pop('invocation')
    assert result == {
        'name': 'foo',
        'id': proj.id
    }



# Copyright (c) 2018 Zim Kalinowski, <zikalino@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


try:
    from ansible.module_utils.ansible_release import __version__ as ANSIBLE_VERSION
except Exception:
    ANSIBLE_VERSION = 'unknown'

try:
    from msrestazure.azure_exceptions import CloudError
    from msrestazure.azure_configuration import AzureConfiguration
    from msrest.service_client import ServiceClient
    from msrest.pipeline import ClientRawResponse
    from msrest.polling import LROPoller
    from msrestazure.polling.arm_polling import ARMPolling
    import uuid
    import json
except ImportError:
    # This is handled in azure_rm_common
    AzureConfiguration = object

ANSIBLE_USER_AGENT = 'Ansible/{0}'.format(ANSIBLE_VERSION)


class GenericRestClientConfiguration(AzureConfiguration):

    def __init__(self, credentials, subscription_id, base_url=None):

        if credentials is None:
            raise ValueError("Parameter 'credentials' must not be None.")
        if subscription_id is None:
            raise ValueError("Parameter 'subscription_id' must not be None.")
        if not base_url:
            base_url = 'https://management.azure.com'

        super(GenericRestClientConfiguration, self).__init__(base_url)

        self.add_user_agent(ANSIBLE_USER_AGENT)

        self.credentials = credentials
        self.subscription_id = subscription_id


class GenericRestClient(object):

    def __init__(self, credentials, subscription_id, base_url=None):
        self.config = GenericRestClientConfiguration(credentials, subscription_id, base_url)
        self._client = ServiceClient(self.config.credentials, self.config)
        self.models = None

    def query(self, url, method, query_parameters, header_parameters, body, expected_status_codes, polling_timeout, polling_interval):
        # Construct and send request
        operation_config = {}

        request = None

        if header_parameters is None:
            header_parameters = {}

        header_parameters['x-ms-client-request-id'] = str(uuid.uuid1())

        if method == 'GET':
            request = self._client.get(url, query_parameters)
        elif method == 'PUT':
            request = self._client.put(url, query_parameters)
        elif method == 'POST':
            request = self._client.post(url, query_parameters)
        elif method == 'HEAD':
            request = self._client.head(url, query_parameters)
        elif method == 'PATCH':
            request = self._client.patch(url, query_parameters)
        elif method == 'DELETE':
            request = self._client.delete(url, query_parameters)
        elif method == 'MERGE':
            request = self._client.merge(url, query_parameters)

        response = self._client.send(request, header_parameters, body, **operation_config)

        if response.status_code not in expected_status_codes:
            exp = CloudError(response)
            exp.request_id = response.headers.get('x-ms-request-id')
            raise exp
        elif response.status_code == 202 and polling_timeout > 0:
            def get_long_running_output(response):
                return response
            poller = LROPoller(self._client,
                               ClientRawResponse(None, response),
                               get_long_running_output,
                               ARMPolling(polling_interval, **operation_config))
            response = self.get_poller_result(poller, polling_timeout)

        return response

    def get_poller_result(self, poller, timeout):
        try:
            poller.wait(timeout=timeout)
            return poller.result()
        except Exception as exc:
            raise



#!/usr/bin/env python
#
# (c) 2018, Red Hat, Inc.
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import sys
import json
import argparse

from ansible.parsing.dataloader import DataLoader
from ansible.module_utils.six import iteritems
from ansible.module_utils._text import to_text
from ansible_collections.community.general.plugins.module_utils.net_tools.nios.api import WapiInventory
from ansible_collections.community.general.plugins.module_utils.net_tools.nios.api import normalize_extattrs, flatten_extattrs


CONFIG_FILES = [
    os.environ.get('INFOBLOX_CONFIG_FILE', ''),
    '/etc/ansible/infoblox.yaml',
    '/etc/ansible/infoblox.yml'
]


def parse_args():
    parser = argparse.ArgumentParser()

    parser.add_argument('--list', action='store_true',
                        help='List host records from NIOS for use in Ansible')

    parser.add_argument('--host',
                        help='List meta data about single host (not used)')

    return parser.parse_args()


def main():
    args = parse_args()

    for config_file in CONFIG_FILES:
        if os.path.exists(config_file):
            break
    else:
        sys.stderr.write('unable to locate config file at /etc/ansible/infoblox.yaml\n')
        sys.exit(-1)

    try:
        loader = DataLoader()
        config = loader.load_from_file(config_file)
        provider = config.get('provider') or {}
        wapi = WapiInventory(provider)
    except Exception as exc:
        sys.stderr.write(to_text(exc))
        sys.exit(-1)

    if args.host:
        host_filter = {'name': args.host}
    else:
        host_filter = {}

    config_filters = config.get('filters')

    if config_filters.get('view') is not None:
        host_filter['view'] = config_filters['view']

    if config_filters.get('extattrs'):
        extattrs = normalize_extattrs(config_filters['extattrs'])
    else:
        extattrs = {}

    hostvars = {}
    inventory = {
        '_meta': {
            'hostvars': hostvars
        }
    }

    return_fields = ['name', 'view', 'extattrs', 'ipv4addrs']

    hosts = wapi.get_object('record:host',
                            host_filter,
                            extattrs=extattrs,
                            return_fields=return_fields)

    if hosts:
        for item in hosts:
            view = item['view']
            name = item['name']

            if view not in inventory:
                inventory[view] = {'hosts': []}

            inventory[view]['hosts'].append(name)

            hostvars[name] = {
                'view': view
            }

            if item.get('extattrs'):
                for key, value in iteritems(flatten_extattrs(item['extattrs'])):
                    if key.startswith('ansible_'):
                        hostvars[name][key] = value
                    else:
                        if 'extattrs' not in hostvars[name]:
                            hostvars[name]['extattrs'] = {}
                        hostvars[name]['extattrs'][key] = value

    sys.stdout.write(json.dumps(inventory, indent=4))
    sys.exit(0)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The module file for eos_ospf_interfaces
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = """
module: eos_ospf_interfaces
version_added: 1.1.0
short_description: OSPF Interfaces Resource Module.
description:
- This module manages OSPF configuration of interfaces on devices running Arista EOS.
author: Gomathi Selvi Srinivasan (@GomathiselviS)
options:
  config:
    description: A list of OSPF configuration for interfaces.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - Name/Identifier of the interface.
        type: str
      address_family:
        description:
        - OSPF settings on the interfaces in address-family context.
        type: list
        elements: dict
        suboptions:
          afi:
            description:
            - Address Family Identifier (AFI) for OSPF settings on the interfaces.
            type: str
            choices: ['ipv4', 'ipv6']
            required: True
          area:
            description:
            - Area associated with interface.
            - Valid only when afi = ipv4.
            type: dict
            suboptions:
              area_id:
                description:
                - Area ID as a decimal or IP address format.
                type: str
                required: True
          authentication_v2:
            description:
            - Authentication settings on the interface.
            - Valid only when afi = ipv4.
            type: dict
            suboptions:
              message_digest:
                description:
                - Use message-digest authentication.
                type: bool
              set:
                description:
                - Enable authentication on the interface.
                type: bool
          authentication_v3:
            description:
            - Authentication settings on the interface.
            - Valid only when afi = ipv6.
            type: dict
            suboptions:
              spi:
                description: IPsec Security Parameter Index.
                type: int
              algorithm:
                description: Encryption alsgorithm.
                type: str
                choices: ["md5", "sha1"]
              keytype:
                description:
                - Specifies if an unencrypted/hidden follows.
                - 0 denotes unencrypted key.
                - 7 denotes hidden key.
                type: str
              passphrase:
                description: Passphrase String for deriving keys for authentication and encryption.
                type: str
              key:
                description: 128 bit MD5 key or 140 bit SHA1 key.
                type: str
          authentication_key:
            description:
            - Configure the authentication key for the interface.
            - Valid only when afi = ipv4.
            type: dict
            suboptions:
              encryption:
                description:
                - 0 Specifies an UNENCRYPTED authentication key will follow.
                - 7 Specifies a proprietry encryption type.`
                type: str
              key:
                description:
                - password (up to 8 chars).
                type: str
          bfd:
            description: Enable BFD.
            type: bool
          cost:
            description:
            - metric associated with interface.
            type: int
          dead_interval:
            description:
            - Time interval to detect a dead router.
            type: int
          encryption_v3:
            description:
            - Authentication settings on the interface.
            - Valid only when afi = ipv6.
            type: dict
            suboptions:
              spi:
                description: IPsec Security Parameter Index.
                type: int
              encryption:
                description: encryption type.
                choices: ["3des-cbc", "aes-128-cbc", "aes-192-cbc", "aes-256-cbc", "null"]
                type: str
              algorithm:
                description: algorithm.
                type: str
                choices: ["md5", "sha1"]
              keytype:
                description:
                - Specifies if an unencrypted/hidden follows.
                - 0 denotes unencrypted key.
                - 7 denotes hidden key.
                type: str
              passphrase:
                description: Passphrase String for deriving keys for authentication and encryption.
                type: str
              key:
                description: key
                type: str
          hello_interval:
            description:
            - Timer interval between transmission of hello packets.
            type: int
          ip_params:
            description:
            - Specify parameters for IPv4/IPv6.
            - Valid only when afi = ipv6.
            type: list
            elements: dict
            suboptions:
              afi:
                description:
                - Address Family Identifier (AFI) for OSPF settings on the interfaces.
                type: str
                choices: ['ipv4', 'ipv6']
                required: True
              area:
                description:
                - Area associated with interface.
                - Valid only when afi = ipv4.
                type: dict
                suboptions:
                  area_id:
                    description:
                    - Area ID as a decimal or IP address format.
                    type: str
                    required: True
              bfd:
                description: Enable BFD.
                type: bool
              cost:
                description:
                - metric associated with interface.
                type: int
              dead_interval:
                description:
                - Time interval to detect a dead router.
                type: int
              hello_interval:
                description:
                - Timer interval between transmission of hello packets.
                type: int
              mtu_ignore:
                description:
                - if True, Disable MTU check for Database Description packets.
                type: bool
              network:
                description:
                - Interface type.
                type: str
              priority:
                description:
                - Interface priority.
                type: int
              retransmit_interval:
                description:
                - LSA retransmission interval.
                type: int
              passive_interface:
                description:
                - Suppress routing updates in an interface.
                type: bool
              transmit_delay:
                description:
                - LSA transmission delay.
                type: int
          message_digest_key:
            description:
            - Message digest authentication password (key) settings.
            type: dict
            suboptions:
              key_id:
                description:
                - Key ID.
                type: int
              encryption:
                description:
                - 0 Specifies an UNENCRYPTED ospf password (key) will follow.
                - 7 Specifies a proprietry encryption type.
                type: str
              key:
                description:
                - Authentication key (upto 16 chars).
                type: str
          mtu_ignore:
            description:
            - if True, Disable MTU check for Database Description packets.
            type: bool
          network:
            description:
            - Interface type.
            type: str
          passive_interface:
            description:
            - Suppress routing updates in an interface.
            - Valid only when afi = ipv6.
            type: bool
          priority:
            description:
            - Interface priority.
            type: int
          retransmit_interval:
            description:
            - LSA retransmission interval.
            type: int
          shutdown:
            description:
            - Shutdown OSPF on this interface.
            type: bool
          transmit_delay:
            description:
            - LSA transmission delay.
            type: int
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the EOS device by
      executing the command B(show running-config | section interface).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str

  state:
    description:
      - The state the configuration should be left in.
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - gathered
    - parsed
    - rendered
    default: merged
"""

EXAMPLES = """

# Using merged

# Before state

# veos(config)#show running-config | section interface | ospf
# veos(config)#

  - name: Merge provided configuration with device configuration
    arista.eos.eos_ospf_interfaces:
      config:
        - name: "Vlan1"
          address_family:
            - afi: "ipv4"
              area:
                area_id: "0.0.0.50"
              cost: 500
              mtu_ignore: True
            - afi: "ipv6"
              dead_interval: 44
              ip_params:
                - afi: "ipv6"
                  mtu_ignore: True
                  network: "point-to-point"
      state: merged

# After State

# veos(config)#show running-config | section interface | ospf
# interface Vlan1
#    ip ospf cost 500
#    ip ospf mtu-ignore
#    ip ospf area 0.0.0.50
#    ospfv3 dead-interval 44
#    ospfv3 ipv6 network point-to-point
#    ospfv3 ipv6 mtu-ignore
# veos(config)#
#
#
# Module Execution:
#
#   "after": [
#         {
#             "name": "Ethernet1"
#         },
#         {
#             "name": "Ethernet2"
#         },
#         {
#             "name": "Management1"
#         },
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv4",
#                     "area": {
#                         "area_id": "0.0.0.50"
#                     },
#                     "cost": 500,
#                     "mtu_ignore": True
#                 },
#                 {
#                     "afi": "ipv6",
#                     "dead_interval": 44,
#                     "ip_params": [
#                         {
#                             "afi": "ipv6",
#                             "mtu_ignore": True,
#                             "network": "point-to-point"
#                         }
#                     ]
#                 }
#             ],
#             "name": "Vlan1"
#         }
#     ],
#     "before": [
#         {
#             "name": "Ethernet1"
#         },
#         {
#             "name": "Ethernet2"
#         },
#         {
#             "name": "Management1"
#         }
#     ],
#     "changed": True,
#     "commands": [
#         "interface Vlan1",
#         "ip ospf area 0.0.0.50",
#         "ip ospf cost 500",
#         "ip ospf mtu-ignore",
#         "ospfv3 dead-interval 44",
#         "ospfv3 ipv6 mtu-ignore",
#         "ospfv3 ipv6 network point-to-point"
#     ],
#

# Using replaced
#---------------

# Before State:

# veos(config)#show running-config | section interface | ospf
# interface Vlan1
#    ip ospf cost 500
#    ip ospf dead-interval 29
#    ip ospf hello-interval 66
#    ip ospf mtu-ignore
#    ip ospf area 0.0.0.50
#    ospfv3 cost 106
#    ospfv3 hello-interval 77
#    ospfv3 dead-interval 44
#    ospfv3 transmit-delay 100
#    ospfv3 ipv4 priority 45
#    ospfv3 ipv4 area 0.0.0.5
#    ospfv3 ipv6 passive-interface
#    ospfv3 ipv6 retransmit-interval 115
#    ospfv3 ipv6 network point-to-point
#    ospfv3 ipv6 mtu-ignore
# !
# interface Vlan2
#    ospfv3 ipv4 hello-interval 45
#    ospfv3 ipv4 retransmit-interval 100
#    ospfv3 ipv4 area 0.0.0.6
# veos(config)#


  - name: Replace device configuration with provided configuration
    arista.eos.eos_ospf_interfaces:
      config:
        - name: "Vlan1"
          address_family:
            - afi: "ipv6"
              cost: 44
              bfd: True
              ip_params:
                - afi: "ipv6"
                  mtu_ignore: True
                  network: "point-to-point"
                  dead_interval: 56
      state: replaced

# After State:

# veos(config)#show running-config | section interface | ospf
# interface Vlan1
#    ospfv3 bfd
#    ospfv3 cost 44
#    no ospfv3 ipv6 passive-interface
#    ospfv3 ipv6 network point-to-point
#    ospfv3 ipv6 mtu-ignore
# !
# interface Vlan2
#    ospfv3 ipv4 hello-interval 45
#    ospfv3 ipv4 retransmit-interval 100
#    ospfv3 ipv4 area 0.0.0.6
# veos(config)#
#
# Module Execution:
#
# "after": [
#         {
#             "name": "Ethernet1"
#         },
#         {
#             "name": "Ethernet2"
#         },
#         {
#             "name": "Management1"
#         },
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv6",
#                     "bfd": True,
#                     "cost": 44,
#                     "ip_params": [
#                         {
#                             "afi": "ipv6",
#                             "mtu_ignore": True,
#                             "network": "point-to-point"
#                         }
#                     ]
#                 }
#             ],
#             "name": "Vlan1"
#         },
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv6",
#                     "ip_params": [
#                         {
#                             "afi": "ipv4",
#                             "area": {
#                                 "area_id": "0.0.0.6"
#                             },
#                             "hello_interval": 45,
#                             "retransmit_interval": 100
#                         }
#                     ]
#                 }
#             ],
#             "name": "Vlan2"
#         }
#     ],
#     "before": [
#         {
#             "name": "Ethernet1"
#         },
#         {
#             "name": "Ethernet2"
#         },
#         {
#             "name": "Management1"
#         },
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv4",
#                     "area": {
#                         "area_id": "0.0.0.50"
#                     },
#                     "cost": 500,
#                     "dead_interval": 29,
#                     "hello_interval": 66,
#                     "mtu_ignore": True
#                 },
#                 {
#                     "afi": "ipv6",
#                     "cost": 106,
#                     "dead_interval": 44,
#                     "hello_interval": 77,
#                     "ip_params": [
#                         {
#                             "afi": "ipv4",
#                             "area": {
#                                 "area_id": "0.0.0.5"
#                             },
#                             "priority": 45
#                         },
#                         {
#                             "afi": "ipv6",
#                             "mtu_ignore": True,
#                             "network": "point-to-point",
#                             "passive_interface": True,
#                             "retransmit_interval": 115
#                         }
#                     ],
#                     "transmit_delay": 100
#                 }
#             ],
#             "name": "Vlan1"
#         },
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv6",
#                     "ip_params": [
#                         {
#                             "afi": "ipv4",
#                             "area": {
#                                 "area_id": "0.0.0.6"
#                             },
#                             "hello_interval": 45,
#                             "retransmit_interval": 100
#                         }
#                     ]
#                 }
#             ],
#             "name": "Vlan2"
#         }
#     ],
#     "changed": True,
#     "commands": [
#         "interface Vlan1",
#         "no ip ospf cost 500",
#         "no ip ospf dead-interval 29",
#         "no ip ospf hello-interval 66",
#         "no ip ospf mtu-ignore",
#         "no ip ospf area 0.0.0.50",
#         "ospfv3 cost 44",
#         "ospfv3 bfd",
#         "ospfv3 authentication ipsec spi 30 md5 passphrase 7 7hl8FV3lZ6H1mAKpjL47hQ==",
#         "no ospfv3 ipv4 priority 45",
#         "no ospfv3 ipv4 area 0.0.0.5",
#         "ospfv3 ipv6 dead-interval 56",
#         "no ospfv3 ipv6 passive-interface",
#         "no ospfv3 ipv6 retransmit-interval 115",
#         "no ospfv3 hello-interval 77",
#         "no ospfv3 dead-interval 44",
#         "no ospfv3 transmit-delay 100"
#     ],
#

# Using overidden:
# ----------------

# Before State:
# veos(config)#show running-config | section interface | ospf
# interface Vlan1
#    ip ospf dead-interval 29
#    ip ospf hello-interval 66
#    ip ospf mtu-ignore
#    ospfv3 bfd
#    ospfv3 cost 106
#    ospfv3 hello-interval 77
#    ospfv3 transmit-delay 100
#    ospfv3 ipv4 priority 45
#    ospfv3 ipv4 area 0.0.0.5
#    ospfv3 ipv6 passive-interface
#    ospfv3 ipv6 dead-interval 56
#    ospfv3 ipv6 retransmit-interval 115
#    ospfv3 ipv6 network point-to-point
#    ospfv3 ipv6 mtu-ignore
# !
# interface Vlan2
#    ospfv3 ipv4 hello-interval 45
#    ospfv3 ipv4 retransmit-interval 100
#    ospfv3 ipv4 area 0.0.0.6
# veos(config)#

  - name: Override device configuration with provided configuration
    arista.eos.eos_ospf_interfaces:
      config:
        - name: "Vlan1"
          address_family:
            - afi: "ipv6"
              cost: 44
              bfd: True
              ip_params:
                - afi: "ipv6"
                  mtu_ignore: True
                  network: "point-to-point"
                  dead_interval: 56
      state: overridden

# After State:

# veos(config)#show running-config | section interface | ospf
# interface Vlan1
#    ospfv3 bfd
#    ospfv3 cost 44
#    no ospfv3 ipv6 passive-interface
#    ospfv3 ipv6 dead-interval 56
#    ospfv3 ipv6 network point-to-point
#    ospfv3 ipv6 mtu-ignore
# veos(config)#
#
#
# Module Execution:
#
#  "after": [
#         {
#             "name": "Ethernet1"
#         },
#         {
#             "name": "Ethernet2"
#         },
#         {
#             "name": "Management1"
#         },
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv6",
#                     "bfd": True,
#                     "cost": 44,
#                     "ip_params": [
#                         {
#                             "afi": "ipv6",
#                             "dead_interval": 56,
#                             "mtu_ignore": True,
#                             "network": "point-to-point"
#                         }
#                     ]
#                 }
#             ],
#             "name": "Vlan1"
#         },
#         {
#             "name": "Vlan2"
#         }
#     ],
#     "before": [
#         {
#             "name": "Ethernet1"
#         },
#         {
#             "name": "Ethernet2"
#         },
#         {
#             "name": "Management1"
#         },
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv4",
#                     "dead_interval": 29,
#                     "hello_interval": 66,
#                     "mtu_ignore": True
#                 },
#                 {
#                     "afi": "ipv6",
#                     "bfd": True,
#                     "cost": 106,
#                     "hello_interval": 77,
#                     "ip_params": [
#                         {
#                             "afi": "ipv4",
#                             "area": {
#                                 "area_id": "0.0.0.5"
#                             },
#                             "priority": 45
#                         },
#                         {
#                             "afi": "ipv6",
#                             "dead_interval": 56,
#                             "mtu_ignore": True,
#                             "network": "point-to-point",
#                             "passive_interface": True,
#                             "retransmit_interval": 115
#                         }
#                     ],
#                     "transmit_delay": 100
#                 }
#             ],
#             "name": "Vlan1"
#         },
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv6",
#                     "ip_params": [
#                         {
#                             "afi": "ipv4",
#                             "area": {
#                                 "area_id": "0.0.0.6"
#                             },
#                             "hello_interval": 45,
#                             "retransmit_interval": 100
#                         }
#                     ]
#                 }
#             ],
#             "name": "Vlan2"
#         }
#     ],
#     "changed": True,
#     "commands": [
#         "interface Vlan2",
#         "no ospfv3 ipv4 hello-interval 45",
#         "no ospfv3 ipv4 retransmit-interval 100",
#         "no ospfv3 ipv4 area 0.0.0.6",
#         "interface Vlan1",
#         "no ip ospf dead-interval 29",
#         "no ip ospf hello-interval 66",
#         "no ip ospf mtu-ignore",
#         "ospfv3 cost 44",
#         "ospfv3 authentication ipsec spi 30 md5 passphrase 7 7hl8FV3lZ6H1mAKpjL47hQ==",
#         "no ospfv3 ipv4 priority 45",
#         "no ospfv3 ipv4 area 0.0.0.5",
#         "no ospfv3 ipv6 passive-interface",
#         "no ospfv3 ipv6 retransmit-interval 115",
#         "no ospfv3 hello-interval 77",
#         "no ospfv3 transmit-delay 100"
#     ],
#

# Using deleted:
#--------------

# before State:

# veos(config)#show running-config | section interface | ospf
# interface Vlan1
#    ip ospf dead-interval 29
#    ip ospf hello-interval 66
#    ip ospf mtu-ignore
#    ospfv3 bfd
#    ospfv3 cost 106
#    ospfv3 hello-interval 77
#    ospfv3 transmit-delay 100
#    ospfv3 ipv4 priority 45
#    ospfv3 ipv4 area 0.0.0.5
#    ospfv3 ipv6 passive-interface
#    ospfv3 ipv6 dead-interval 56
#    ospfv3 ipv6 retransmit-interval 115
#    ospfv3 ipv6 network point-to-point
#    ospfv3 ipv6 mtu-ignore
# !
# interface Vlan2
#    ospfv3 ipv4 hello-interval 45
#    ospfv3 ipv4 retransmit-interval 100
#    ospfv3 ipv4 area 0.0.0.6
# veos(config)#

  - name: Delete device configuration
    arista.eos.eos_ospf_interfaces:
      config:
        - name: "Vlan1"
      state: deleted

# After State:

# veos#show running-config | section interface | ospf
# interface Vlan2
#    ospfv3 ipv4 hello-interval 45
#    ospfv3 ipv4 retransmit-interval 100
#    ospfv3 ipv4 area 0.0.0.6
#
# Module Execution:
#
# "after": [
#         {
#             "name": "Ethernet1"
#         },
#         {
#             "name": "Ethernet2"
#         },
#         {
#             "name": "Management1"
#         },
#         {
#             "name": "Vlan1"
#         },
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv6",
#                     "ip_params": [
#                         {
#                             "afi": "ipv4",
#                             "area": {
#                                 "area_id": "0.0.0.6"
#                             },
#                             "hello_interval": 45,
#                             "retransmit_interval": 100
#                         }
#                     ]
#                 }
#             ],
#             "name": "Vlan2"
#         }
#     ],
#     "before": [
#         {
#             "name": "Ethernet1"
#         },
#         {
#             "name": "Ethernet2"
#         },
#         {
#             "name": "Management1"
#         },
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv4",
#                     "dead_interval": 29,
#                     "hello_interval": 66,
#                     "mtu_ignore": True
#                 },
#                 {
#                     "afi": "ipv6",
#                     "bfd": True,
#                     "cost": 106,
#                     "hello_interval": 77,
#                     "ip_params": [
#                         {
#                             "afi": "ipv4",
#                             "area": {
#                                 "area_id": "0.0.0.5"
#                             },
#                             "priority": 45
#                         },
#                         {
#                             "afi": "ipv6",
#                             "dead_interval": 56,
#                             "mtu_ignore": True,
#                             "network": "point-to-point",
#                             "passive_interface": True,
#                             "retransmit_interval": 115
#                         }
#                     ],
#                     "transmit_delay": 100
#                 }
#             ],
#             "name": "Vlan1"
#         },
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv6",
#                     "ip_params": [
#                         {
#                             "afi": "ipv4",
#                             "area": {
#                                 "area_id": "0.0.0.6"
#                             },
#                             "hello_interval": 45,
#                             "retransmit_interval": 100
#                         }
#                     ]
#                 }
#             ],
#             "name": "Vlan2"
#         }
#     ],
#     "changed": True,
#     "commands": [
#         "interface Vlan1",
#         "no ip ospf dead-interval 29",
#         "no ip ospf hello-interval 66",
#         "no ip ospf mtu-ignore",
#         "no ospfv3 bfd",
#         "no ospfv3 cost 106",
#         "no ospfv3 hello-interval 77",
#         "no ospfv3 transmit-delay 100",
#         "no ospfv3 ipv4 priority 45",
#         "no ospfv3 ipv4 area 0.0.0.5",
#         "no ospfv3 ipv6 passive-interface",
#         "no ospfv3 ipv6 dead-interval 56",
#         "no ospfv3 ipv6 retransmit-interval 115",
#         "no ospfv3 ipv6 network point-to-point",
#         "no ospfv3 ipv6 mtu-ignore"
#     ],
#

# Using parsed:
# ------------

# parsed.cfg:
# ----------

# interface Vlan1
#    ip ospf dead-interval 29
#    ip ospf hello-interval 66
#    ip ospf mtu-ignore
#    ip ospf cost 500
#    ospfv3 bfd
#    ospfv3 cost 106
#    ospfv3 hello-interval 77
#    ospfv3 transmit-delay 100
#    ospfv3 ipv4 priority 45
#    ospfv3 ipv4 area 0.0.0.5
#    ospfv3 ipv6 passive-interface
#    ospfv3 ipv6 dead-interval 56
#    ospfv3 ipv6 retransmit-interval 115
#    ospfv3 ipv6 network point-to-point
#    ospfv3 ipv6 mtu-ignore
# !
# interface Vlan2
#    ospfv3 ipv4 hello-interval 45
#    ospfv3 ipv4 retransmit-interval 100
#    ospfv3 ipv4 area 0.0.0.6
#

  - name: parse configs
    arista.eos.eos_ospf_interfaces:
      running_config: "{{ lookup('file', './parsed.cfg') }}"
      state: parsed

# Module Execution:
# "parsed": [
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv4",
#                     "cost": 500,
#                     "dead_interval": 29,
#                     "hello_interval": 66,
#                     "mtu_ignore": True
#                 },
#                 {
#                     "afi": "ipv6",
#                     "bfd": True,
#                     "cost": 106,
#                     "hello_interval": 77,
#                     "ip_params": [
#                         {
#                             "afi": "ipv4",
#                             "area": {
#                                 "area_id": "0.0.0.5"
#                             },
#                             "priority": 45
#                         },
#                         {
#                             "afi": "ipv6",
#                             "dead_interval": 56,
#                             "mtu_ignore": True,
#                             "network": "point-to-point",
#                             "passive_interface": True,
#                             "retransmit_interval": 115
#                         }
#                     ],
#                     "transmit_delay": 100
#                 }
#             ],
#             "name": "Vlan1"
#         },
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv6",
#                     "ip_params": [
#                         {
#                             "afi": "ipv4",
#                             "area": {
#                                 "area_id": "0.0.0.6"
#                             },
#                             "hello_interval": 45,
#                             "retransmit_interval": 100
#                         }
#                     ]
#                 }
#             ],
#             "name": "Vlan2"
#         }
#     ]

# Using gathered:

# Device COnfig:

# veos#show running-config | section interface | ospf
# interface Vlan1
#    ip ospf cost 500
#    ip ospf dead-interval 29
#    ip ospf hello-interval 66
#    ip ospf mtu-ignore
#    ip ospf area 0.0.0.50
#    ospfv3 cost 106
#    ospfv3 hello-interval 77
#    ospfv3 transmit-delay 100
#    ospfv3 ipv4 priority 45
#    ospfv3 ipv4 area 0.0.0.5
#    ospfv3 ipv6 passive-interface
#    ospfv3 ipv6 dead-interval 56
#    ospfv3 ipv6 retransmit-interval 115
#    ospfv3 ipv6 network point-to-point
#    ospfv3 ipv6 mtu-ignore
# !
# interface Vlan2
#    ospfv3 ipv4 hello-interval 45
#    ospfv3 ipv4 retransmit-interval 100
#    ospfv3 ipv4 area 0.0.0.6
# veos#

  - name: gather configs
    arista.eos.eos_ospf_interfaces:
      state: gathered

# Module Execution:
#
#  "gathered": [
#         {
#             "name": "Ethernet1"
#         },
#         {
#             "name": "Ethernet2"
#         },
#         {
#             "name": "Management1"
#         },
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv4",
#                     "area": {
#                         "area_id": "0.0.0.50"
#                     },
#                     "cost": 500,
#                     "dead_interval": 29,
#                     "hello_interval": 66,
#                     "mtu_ignore": True
#                 },
#                 {
#                     "afi": "ipv6",
#                     "cost": 106,
#                     "hello_interval": 77,
#                     "ip_params": [
#                         {
#                             "afi": "ipv4",
#                             "area": {
#                                 "area_id": "0.0.0.5"
#                             },
#                             "priority": 45
#                         },
#                         {
#                             "afi": "ipv6",
#                             "dead_interval": 56,
#                             "mtu_ignore": True,
#                             "network": "point-to-point",
#                             "passive_interface": True,
#                             "retransmit_interval": 115
#                         }
#                     ],
#                     "transmit_delay": 100
#                 }
#             ],
#             "name": "Vlan1"
#         },
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv6",
#                     "ip_params": [
#                         {
#                             "afi": "ipv4",
#                             "area": {
#                                 "area_id": "0.0.0.6"
#                             },
#                             "hello_interval": 45,
#                             "retransmit_interval": 100
#                         }
#                     ]
#                 }
#             ],
#             "name": "Vlan2"
#         }
#     ],
#


# Using rendered:
# --------------

  - name: Render provided configuration
    arista.eos.eos_ospf_interfaces:
      config:
        - name: "Vlan1"
          address_family:
            - afi: "ipv4"
              dead_interval: 29
              mtu_ignore: True
              hello_interval: 66
            - afi: "ipv6"
              hello_interval: 77
              cost : 106
              transmit_delay: 100
              ip_params:
                - afi: "ipv6"
                  retransmit_interval: 115
                  dead_interval: 56
                  passive_interface: True
                - afi: "ipv4"
                  area:
                    area_id: "0.0.0.5"
                  priority: 45
        - name: "Vlan2"
          address_family:
            - afi: "ipv6"
              ip_params:
                - afi: "ipv4"
                  area:
                    area_id: "0.0.0.6"
                  hello_interval: 45
                  retransmit_interval: 100
            - afi: "ipv4"
              message_digest_key:
                key_id: 200
                encryption: 7
                key: "hkdfhtu=="

      state: rendered

# Module Execution:
#
# "rendered": [
#         "interface Vlan1",
#         "ip ospf dead-interval 29",
#         "ip ospf mtu-ignore",
#         "ip ospf hello-interval 66",
#         "ospfv3 hello-interval 77",
#         "ospfv3 cost 106",
#         "ospfv3 transmit-delay 100",
#         "ospfv3 ipv4 area 0.0.0.5",
#         "ospfv3 ipv4 priority 45",
#         "ospfv3 ipv6 retransmit-interval 115",
#         "ospfv3 ipv6 dead-interval 56",
#         "ospfv3 ipv6 passive-interface",
#         "interface Vlan2",
#         "ip ospf message-digest-key 200 md5 7 hkdfhtu==",
#         "ospfv3 ipv4 area 0.0.0.6",
#         "ospfv3 ipv4 hello-interval 45",
#         "ospfv3 ipv4 retransmit-interval 100"
#     ]
#

"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.ospf_interfaces.ospf_interfaces import (
    Ospf_interfacesArgs,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.config.ospf_interfaces.ospf_interfaces import (
    Ospf_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Ospf_interfacesArgs.argument_spec,
        mutually_exclusive=[],
        required_if=[],
        supports_check_mode=False,
    )

    result = Ospf_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright: (c) 2019-2020, Andrew Klaus <andrewklaus@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = r'''
---
module: syspatch

short_description: Manage OpenBSD system patches


description:
    - "Manage OpenBSD system patches using syspatch."

options:
    apply:
        type: bool
        description:
            - Apply all available system patches.
            - By default, apply all patches.
            - Deprecated. Will be removed in community.general 3.0.0.
        default: yes
    revert:
        description:
            - Revert system patches.
        type: str
        choices: [ all, one ]

author:
    - Andrew Klaus (@precurse)
'''

EXAMPLES = '''
- name: Apply all available system patches
  community.general.syspatch:
    apply: true

- name: Revert last patch
  community.general.syspatch:
    revert: one

- name: Revert all patches
  community.general.syspatch:
    revert: all

# NOTE: You can reboot automatically if a patch requires it:
- name: Apply all patches and store result
  community.general.syspatch:
    apply: true
  register: syspatch

- name: Reboot if patch requires it
  ansible.builtin.reboot:
  when: syspatch.reboot_needed
'''

RETURN = r'''
rc:
  description: The command return code (0 means success)
  returned: always
  type: int
stdout:
  description: syspatch standard output.
  returned: always
  type: str
  sample: "001_rip6cksum"
stderr:
  description: syspatch standard error.
  returned: always
  type: str
  sample: "syspatch: need root privileges"
reboot_needed:
  description: Whether or not a reboot is required after an update.
  returned: always
  type: bool
  sample: True
'''

from ansible.module_utils.basic import AnsibleModule


def run_module():
    # define available arguments/parameters a user can pass to the module
    module_args = dict(
        apply=dict(type='bool', default=True, removed_in_version='3.0.0', removed_from_collection='community.general'),
        revert=dict(type='str', choices=['all', 'one'])
    )

    module = AnsibleModule(
        argument_spec=module_args,
        supports_check_mode=True,
        required_one_of=[['apply', 'revert']]
    )

    result = syspatch_run(module)

    module.exit_json(**result)


def syspatch_run(module):
    cmd = module.get_bin_path('syspatch', True)
    changed = False
    reboot_needed = False
    warnings = []

    # Set safe defaults for run_flag and check_flag
    run_flag = ['-c']
    check_flag = ['-c']
    if module.params['revert']:
        check_flag = ['-l']

        if module.params['revert'] == 'all':
            run_flag = ['-R']
        else:
            run_flag = ['-r']
    elif module.params['apply']:
        check_flag = ['-c']
        run_flag = []

    # Run check command
    rc, out, err = module.run_command([cmd] + check_flag)

    if rc != 0:
        module.fail_json(msg="Command %s failed rc=%d, out=%s, err=%s" % (cmd, rc, out, err))

    if len(out) > 0:
        # Changes pending
        change_pending = True
    else:
        # No changes pending
        change_pending = False

    if module.check_mode:
        changed = change_pending
    elif change_pending:
        rc, out, err = module.run_command([cmd] + run_flag)

        # Workaround syspatch ln bug:
        # http://openbsd-archive.7691.n7.nabble.com/Warning-applying-latest-syspatch-td354250.html
        if rc != 0 and err != 'ln: /usr/X11R6/bin/X: No such file or directory\n':
            module.fail_json(msg="Command %s failed rc=%d, out=%s, err=%s" % (cmd, rc, out, err))
        elif out.lower().find('create unique kernel') >= 0:
            # Kernel update applied
            reboot_needed = True
        elif out.lower().find('syspatch updated itself') >= 0:
            warnings.append('Syspatch was updated. Please run syspatch again.')

        # If no stdout, then warn user
        if len(out) == 0:
            warnings.append('syspatch had suggested changes, but stdout was empty.')

        changed = True
    else:
        changed = False

    return dict(
        changed=changed,
        reboot_needed=reboot_needed,
        rc=rc,
        stderr=err,
        stdout=out,
        warnings=warnings
    )


def main():
    run_module()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2014, Trond Hindenes <trond@hindenes.com>
# Copyright: (c) 2018, Ansible Project
# Copyright: (c) 2020, Chocolatey Software
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

# this is a windows documentation stub.  actual code lives in the .ps1
# file of the same name

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: win_chocolatey
version_added: '1.9'
short_description: Manage packages using chocolatey
description:
- Manage packages using Chocolatey.
- If Chocolatey is missing from the system, the module will install it.
requirements:
- chocolatey >= 0.10.5 (will be upgraded if older)
options:
  allow_empty_checksums:
    description:
    - Allow empty checksums to be used for downloaded resource from non-secure
      locations.
    - Use M(win_chocolatey_feature) with the name C(allowEmptyChecksums) to
      control this option globally.
    type: bool
    default: no
    version_added: '2.2'
  allow_multiple:
    description:
    - Allow the installation of multiple packages when I(version) is specified.
    - Having multiple packages at different versions can cause issues if the
      package doesn't support this. Use at your own risk.
    type: bool
    default: no
    version_added: '2.8'
  allow_prerelease:
    description:
    - Allow the installation of pre-release packages.
    - If I(state) is C(latest), the latest pre-release package will be
      installed.
    type: bool
    default: no
    version_added: '2.6'
  architecture:
    description:
    - Force Chocolatey to install the package of a specific process
      architecture.
    - When setting C(x86), will ensure Chocolatey installs the x86 package
      even when on an x64 bit OS.
    type: str
    choices: [ default, x86 ]
    default: default
    version_added: '2.7'
  force:
    description:
    - Forces the install of a package, even if it already is installed.
    - Using I(force) will cause Ansible to always report that a change was
      made.
    type: bool
    default: no
  ignore_checksums:
    description:
    - Ignore the checksums provided by the package.
    - Use M(win_chocolatey_feature) with the name C(checksumFiles) to control
      this option globally.
    type: bool
    default: no
    version_added: '2.2'
  ignore_dependencies:
    description:
    - Ignore dependencies, only install/upgrade the package itself.
    type: bool
    default: no
    version_added: '2.1'
  install_args:
    description:
    - Arguments to pass to the native installer.
    - These are arguments that are passed directly to the installer the
      Chocolatey package runs, this is generally an advanced option.
    type: str
    version_added: '2.1'
  name:
    description:
    - Name of the package(s) to be installed.
    - Set to C(all) to run the action on all the installed packages.
    type: list
    required: yes
  override_args:
    description:
    - Override arguments of native installer with arguments provided by user.
    - Should install arguments be used exclusively without appending
      to current package passed arguments.
    type: bool
    version_added: '2.10'
  package_params:
    description:
    - Parameters to pass to the package.
    - These are parameters specific to the Chocolatey package and are generally
      documented by the package itself.
    - Before Ansible 2.7, this option was just I(params).
    type: str
    version_added: '2.1'
    aliases: [ params ]
  pinned:
    description:
    - Whether to pin the Chocolatey package or not.
    - If omitted then no checks on package pins are done.
    - Will pin/unpin the specific version if I(version) is set.
    - Will pin the latest version of a package if C(yes), I(version) is not set
      and and no pin already exists.
    - Will unpin all versions of a package if C(no) and I(version) is not set.
    - This is ignored when C(state=absent).
    type: bool
    version_added: '2.8'
  proxy_url:
    description:
    - Proxy URL used to install chocolatey and the package.
    - Use M(win_chocolatey_config) with the name C(proxy) to control this
      option globally.
    type: str
    version_added: '2.4'
  proxy_username:
    description:
    - Proxy username used to install Chocolatey and the package.
    - Before Ansible 2.7, users with double quote characters C(") would need to
      be escaped with C(\) beforehand. This is no longer necessary.
    - Use M(win_chocolatey_config) with the name C(proxyUser) to control this
      option globally.
    type: str
    version_added: '2.4'
  proxy_password:
    description:
    - Proxy password used to install Chocolatey and the package.
    - This value is exposed as a command argument and any privileged account
      can see this value when the module is running Chocolatey, define the
      password on the global config level with M(win_chocolatey_config) with
      name C(proxyPassword) to avoid this.
    type: str
    version_added: '2.4'
  skip_scripts:
    description:
    - Do not run I(chocolateyInstall.ps1) or I(chocolateyUninstall.ps1) scripts
      when installing a package.
    type: bool
    default: no
    version_added: '2.4'
  source:
    description:
    - Specify the source to retrieve the package from.
    - Use M(win_chocolatey_source) to manage global sources.
    - This value can either be the URL to a Chocolatey feed, a path to a folder
      containing C(.nupkg) packages or the name of a source defined by
      M(win_chocolatey_source).
    - This value is also used when Chocolatey is not installed as the location
      of the install.ps1 script and only supports URLs for this case.
    type: str
  source_username:
    description:
    - A username to use with I(source) when accessing a feed that requires
      authentication.
    - It is recommended you define the credentials on a source with
      M(win_chocolatey_source) instead of passing it per task.
    type: str
    version_added: '2.7'
  source_password:
    description:
    - The password for I(source_username).
    - This value is exposed as a command argument and any privileged account
      can see this value when the module is running Chocolatey, define the
      credentials with a source with M(win_chocolatey_source) to avoid this.
    type: str
    version_added: '2.7'
  state:
    description:
    - State of the package on the system.
    - When C(absent), will ensure the package is not installed.
    - When C(present), will ensure the package is installed.
    - When C(downgrade), will allow Chocolatey to downgrade a package if
      I(version) is older than the installed version.
    - When C(latest), will ensure the package is installed to the latest
      available version.
    - When C(reinstalled), will uninstall and reinstall the package.
    type: str
    choices: [ absent, downgrade, latest, present, reinstalled ]
    default: present
  timeout:
    description:
    - The time to allow chocolatey to finish before timing out.
    type: int
    default: 2700
    version_added: '2.3'
    aliases: [ execution_timeout ]
  validate_certs:
    description:
    - Used when downloading the Chocolatey install script if Chocolatey is not
      already installed, this does not affect the Chocolatey package install
      process.
    - When C(no), no SSL certificates will be validated.
    - This should only be used on personally controlled sites using self-signed
      certificate.
    type: bool
    default: yes
    version_added: '2.7'
  version:
    description:
    - Specific version of the package to be installed.
    - When I(state) is set to C(absent), will uninstall the specific version
      otherwise all versions of that package will be removed.
    - If a different version of package is installed, I(state) must be C(latest)
      or I(force) set to C(yes) to install the desired version.
    - Provide as a string (e.g. C('6.1')), otherwise it is considered to be
      a floating-point number and depending on the locale could become C(6,1),
      which will cause a failure.
    - If I(name) is set to C(chocolatey) and Chocolatey is not installed on the
      host, this will be the version of Chocolatey that is installed. You can
      also set the C(chocolateyVersion) environment var.
    type: str
notes:
- This module will install or upgrade Chocolatey when needed.
- When using verbosity 2 or less (C(-vv)) the C(stdout) output will be restricted.
  When using verbosity 4 (C(-vvvv)) the C(stdout) output will be more verbose.
  When using verbosity 5 (C(-vvvvv)) the C(stdout) output will include debug output.
- Some packages, like hotfixes or updates need an interactive user logon in
  order to install. You can use C(become) to achieve this, see
  :ref:`become_windows`.
  Even if you are connecting as local Administrator, using C(become) to
  become Administrator will give you an interactive user logon, see examples
  below.
- If C(become) is unavailable, use M(win_hotfix) to install hotfixes instead
  of M(win_chocolatey) as M(win_hotfix) avoids using C(wusa.exe) which cannot
  be run without C(become).
seealso:
- module: win_chocolatey_config
- module: win_chocolatey_facts
- module: win_chocolatey_feature
- module: win_chocolatey_source
- module: win_feature
- module: win_hotfix
  description: Use when C(become) is unavailable, to avoid using C(wusa.exe).
- module: win_package
- module: win_updates
- name: Chocolatey website
  description: More information about the Chocolatey tool.
  link: http://chocolatey.org/
- name: Chocolatey packages
  description: An overview of the available Chocolatey packages.
  link: http://chocolatey.org/packages
- ref: become_windows
  description: Some packages, like hotfixes or updates need an interactive user logon
    in order to install. You can use C(become) to achieve this.
author:
- Trond Hindenes (@trondhindenes)
- Peter Mounce (@petemounce)
- Pepe Barbe (@elventear)
- Adam Keech (@smadam813)
- Pierre Templier (@ptemplier)
- Jordan Borean (@jborean93)
'''

# TODO:
# * Better parsing when a package has dependencies - currently fails
# * Time each item that is run
# * Support 'changed' with gems - would require shelling out to `gem list` first and parsing, kinda defeating the point of using chocolatey.
# * Version provided not as string might be translated to 6,6 depending on Locale (results in errors)

EXAMPLES = r'''
- name: Install git
  win_chocolatey:
    name: git
    state: present

- name: Upgrade installed packages
  win_chocolatey:
    name: all
    state: latest

- name: Install notepadplusplus version 6.6
  win_chocolatey:
    name: notepadplusplus
    version: '6.6'

- name: Install notepadplusplus 32 bit version
  win_chocolatey:
    name: notepadplusplus
    architecture: x86

- name: Install git from specified repository
  win_chocolatey:
    name: git
    source: https://someserver/api/v2/

- name: Install git from a pre configured source (win_chocolatey_source)
  win_chocolatey:
    name: git
    source: internal_repo

- name: Ensure Chocolatey itself is installed and use internal repo as source
  win_chocolatey:
    name: chocolatey
    source: http://someserver/chocolatey

- name: Uninstall git
  win_chocolatey:
    name: git
    state: absent

- name: Install multiple packages
  win_chocolatey:
    name:
    - procexp
    - putty
    - windirstat
    state: present

- name: Install multiple packages sequentially
  win_chocolatey:
    name: '{{ item }}'
    state: present
  loop:
  - procexp
  - putty
  - windirstat

- name: Uninstall multiple packages
  win_chocolatey:
    name:
    - procexp
    - putty
    - windirstat
    state: absent

- name: Install curl using proxy
  win_chocolatey:
    name: curl
    proxy_url: http://proxy-server:8080/
    proxy_username: joe
    proxy_password: p@ssw0rd

- name: Install a package that requires 'become'
  win_chocolatey:
    name: officepro2013
  become: yes
  become_user: Administrator
  become_method: runas

- name: install and pin Notepad++ at 7.6.3
  win_chocolatey:
    name: notepadplusplus
    version: 7.6.3
    pinned: yes
    state: present

- name: remove all pins for Notepad++ on all versions
  win_chocolatey:
    name: notepadplusplus
    pinned: no
    state: present
'''

RETURN = r'''
command:
  description: The full command used in the chocolatey task.
  returned: changed
  type: str
  sample: choco.exe install -r --no-progress -y sysinternals --timeout 2700 --failonunfound
rc:
  description: The return code from the chocolatey task.
  returned: always
  type: int
  sample: 0
stdout:
  description: The stdout from the chocolatey task. The verbosity level of the
    messages are affected by Ansible verbosity setting, see notes for more
    details.
  returned: changed
  type: str
  sample: Chocolatey upgraded 1/1 packages.
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, Cindy Zhao (@cizhao) <cizhao@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: mso_schema_template_anp_epg_selector
short_description: Manage EPG selector in schema templates
description:
- Manage EPG selector in schema templates on Cisco ACI Multi-Site.
author:
- Cindy Zhao (@cizhao)
options:
  schema:
    description:
    - The name of the schema.
    type: str
    required: yes
  template:
    description:
    - The name of the template to change.
    type: str
    required: yes
  anp:
    description:
    - The name of the ANP.
    type: str
    required: yes
  epg:
    description:
    - The name of the EPG to manage.
    type: str
    required: yes
  selector:
    description:
    - The name of the selector.
    type: str
  expressions:
    description:
    - Expressions associated to this selector.
    type: list
    elements: dict
    suboptions:
      type:
        description:
        - The name of the expression.
        required: true
        type: str
        aliases: [ tag ]
      operator:
        description:
        - The operator associated to the expression.
        required: true
        type: str
        choices: [ not_in, in, equals, not_equals, has_key, does_not_have_key ]
      value:
        description:
        - The value associated to the expression.
        - If the operator is in or not_in, the value should be a comma separated str.
        type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
seealso:
- module: cisco.mso.mso_schema_template_anp_epg
extends_documentation_fragment: cisco.mso.modules
'''

EXAMPLES = r'''
- name: Add a selector to an EPG
  cisco.mso.mso_schema_template_anp_epg_selector:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    selector: selector_1
    expressions:
      - type: expression_1
        operator: in
        value: test
    state: present
  delegate_to: localhost

- name: Remove a Selector
  cisco.mso.mso_schema_template_anp_epg_selector:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    selector: selector_1
    state: absent
  delegate_to: localhost

- name: Query a specific Selector
  cisco.mso.mso_schema_template_anp_epg_selector:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    selector: selector_1
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all Selectors
  cisco.mso.mso_schema_template_anp_epg_selector:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    state: query
  delegate_to: localhost
  register: query_result
'''

RETURN = r'''
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, mso_expression_spec

EXPRESSION_KEYS = {
    'not_in': 'notIn',
    'not_equals': 'notEquals',
    'has_key': 'keyExist',
    'does_not_have_key': 'keyNotExist',
    'in': 'in',
    'equals': 'equals',
}


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type='str', required=True),
        template=dict(type='str', required=True),
        anp=dict(type='str', required=True),
        epg=dict(type='str', required=True),
        selector=dict(type='str'),
        expressions=dict(type='list', elements='dict', options=mso_expression_spec()),
        state=dict(type='str', default='present', choices=['absent', 'present', 'query']),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ['state', 'absent', ['selector']],
            ['state', 'present', ['selector']],
        ],
    )

    schema = module.params.get('schema')
    template = module.params.get('template').replace(' ', '')
    anp = module.params.get('anp')
    epg = module.params.get('epg')
    selector = module.params.get('selector')
    expressions = module.params.get('expressions')
    state = module.params.get('state')

    mso = MSOModule(module)

    # Get schema_id
    schema_obj = mso.get_obj('schemas', displayName=schema)
    if not schema_obj:
        mso.fail_json(msg="Provided schema '{0}' does not exist".format(schema))

    schema_path = 'schemas/{id}'.format(**schema_obj)

    # Get template
    templates = [t.get('name') for t in schema_obj.get('templates')]
    if template not in templates:
        mso.fail_json(msg="Provided template '{template}' does not exist. Existing templates: {templates}".format(template=template,
                                                                                                                  templates=', '.join(templates)))
    template_idx = templates.index(template)

    # Get ANP
    anps = [a.get('name') for a in schema_obj.get('templates')[template_idx]['anps']]
    if anp not in anps:
        mso.fail_json(msg="Provided anp '{anp}' does not exist. Existing anps: {anps}".format(anp=anp, anps=', '.join(anps)))
    anp_idx = anps.index(anp)

    # Get EPG
    epgs = [e.get('name') for e in schema_obj.get('templates')[template_idx]['anps'][anp_idx]['epgs']]
    if epg not in epgs:
        mso.fail_json(msg="Provided epg '{epg}' does not exist. Existing epgs: {epgs}".format(epg=epg, epgs=', '.join(epgs)))
    epg_idx = epgs.index(epg)

    # Get Selector
    if selector and " " in selector:
        mso.fail_json(msg="There should not be any space in selector name.")
    selectors = [s.get('name') for s in schema_obj.get('templates')[template_idx]['anps'][anp_idx]['epgs'][epg_idx]['selectors']]
    if selector in selectors:
        selector_idx = selectors.index(selector)
        selector_path = '/templates/{0}/anps/{1}/epgs/{2}/selectors/{3}'.format(template, anp, epg, selector_idx)
        mso.existing = schema_obj.get('templates')[template_idx]['anps'][anp_idx]['epgs'][epg_idx]['selectors'][selector_idx]

    if state == 'query':
        if selector is None:
            mso.existing = schema_obj.get('templates')[template_idx]['anps'][anp_idx]['epgs'][epg_idx]['selectors']
        elif not mso.existing:
            mso.fail_json(msg="Selector '{selector}' not found".format(selector=selector))
        mso.exit_json()

    selectors_path = '/templates/{0}/anps/{1}/epgs/{2}/selectors/-'.format(template, anp, epg)
    ops = []

    mso.previous = mso.existing
    if state == 'absent':
        mso.sent = mso.existing = {}
        ops.append(dict(op='remove', path=selector_path))

    elif state == 'present':
        # Get expressions
        all_expressions = []
        if expressions:
            for expression in expressions:
                tag = expression.get('type')
                operator = expression.get('operator')
                value = expression.get('value')
                if " " in tag:
                    mso.fail_json(msg="There should not be any space in 'type' attribute of expression '{0}'".format(tag))
                if operator in ["has_key", "does_not_have_key"] and value:
                    mso.fail_json(
                        msg="Attribute 'value' is not supported for operator '{0}' in expression '{1}'".format(operator, tag))
                if operator in ["not_in", "in", "equals", "not_equals"] and not value:
                    mso.fail_json(
                        msg="Attribute 'value' needed for operator '{0}' in expression '{1}'".format(operator, tag))
                all_expressions.append(dict(
                    key='Custom:' + tag,
                    operator=EXPRESSION_KEYS.get(operator),
                    value=value,
                ))

        payload = dict(
            name=selector,
            expressions=all_expressions,
        )

        mso.sanitize(payload, collate=True)

        if mso.existing:
            ops.append(dict(op='replace', path=selector_path, value=mso.sent))
        else:
            ops.append(dict(op='add', path=selectors_path, value=mso.sent))

        mso.existing = mso.proposed

    if not module.check_mode and mso.existing != mso.previous:
        mso.request(schema_path, method='PATCH', data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright: (c) 2018, Juergen Wiebe <wiebe@e-spirit.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
---
module: utm_network_interface_address_info

author:
    - Juergen Wiebe (@steamx)

short_description: Get info for a network/interface_address object

description:
    - Get info for a network/interface_address object in SOPHOS UTM.


options:
    name:
        type: str
        description:
          - The name of the object. Will be used to identify the entry
        required: true

extends_documentation_fragment:
- community.general.utm

'''

EXAMPLES = """
- name: Get network interface address info
  utm_proxy_interface_address_info:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestNetworkInterfaceAddress
"""

RETURN = """
result:
    description: The utm object that was created
    returned: success
    type: complex
    contains:
        _ref:
            description: The reference name of the object
            type: str
        _locked:
            description: Whether or not the object is currently locked
            type: bool
        _type:
            description: The type of the object
            type: str
        name:
            description: The name of the object
            type: str
        address:
             description: The ip4 address of the network/interface_address object
             type: str
        address6:
             description: The ip6 address of the network/interface_address object
             type: str
        comment:
            description: The comment string
            type: str
        resolved:
             description: Whether or not the object is resolved
             type: bool
        resolved6:
             description: Whether or not the object is resolved
             type: bool
"""

from ansible_collections.community.general.plugins.module_utils.utm_utils import UTM, UTMModule
from ansible.module_utils._text import to_native


def main():
    endpoint = "network/interface_address"
    key_to_check_for_changes = []
    module = UTMModule(
        argument_spec=dict(
            name=dict(type='str', required=True)
        )
    )
    try:
        UTM(module, endpoint, key_to_check_for_changes, info_only=True).execute()
    except Exception as e:
        module.fail_json(msg=to_native(e))


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright: (c) 2020, Christian Wollinger <cwollinger@web.de>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
---
module: ipwcli_dns

short_description: Manage DNS Records for Ericsson IPWorks via ipwcli

version_added: '0.2.0'

description:
    - "Manage DNS records for the Ericsson IPWorks DNS server. The module will use the ipwcli to deploy the DNS records."

requirements:
   - ipwcli (installed on Ericsson IPWorks)

notes:
    - To make the DNS record changes effective, you need to run C(update dnsserver) on the ipwcli.

options:
    dnsname:
        description:
            - Name of the record.
        required: true
        type: str
    type:
        description:
            - Type of the record.
        required: true
        type: str
        choices: [ NAPTR, SRV, A, AAAA ]
    container:
        description:
            - Sets the container zone for the record.
        required: true
        type: str
    address:
        description:
            - The IP address for the A or AAAA record.
            - Required for C(type=A) or C(type=AAAA)
        type: str
    ttl:
        description:
            - Sets the TTL of the record.
        type: int
        default: 3600
    state:
        description:
            - Whether the record should exist or not.
        type: str
        choices: [ absent, present ]
        default: present
    priority:
        description:
            - Sets the priority of the SRV record.
        type: int
        default: 10
    weight:
        description:
            - Sets the weight of the SRV record.
        type: int
        default: 10
    port:
        description:
            - Sets the port of the SRV record.
            - Required for C(type=SRV)
        type: int
    target:
        description:
            - Sets the target of the SRV record.
            - Required for C(type=SRV)
        type: str
    order:
        description:
            - Sets the order of the NAPTR record.
            - Required for C(type=NAPTR)
        type: int
    preference:
        description:
            - Sets the preference of the NAPTR record.
            - Required for C(type=NAPTR)
        type: int
    flags:
        description:
            - Sets one of the possible flags of NAPTR record.
            - Required for C(type=NAPTR)
        type: str
        choices: ['S', 'A', 'U', 'P']
    service:
        description:
            - Sets the service of the NAPTR record.
            - Required for C(type=NAPTR)
        type: str
    replacement:
        description:
            - Sets the replacement of the NAPTR record.
            - Required for C(type=NAPTR)
        type: str
    username:
        description:
            - Username to login on ipwcli.
        type: str
        required: true
    password:
        description:
            - Password to login on ipwcli.
        type: str
        required: true

author:
    - Christian Wollinger (@cwollinger)
'''

EXAMPLES = '''
- name: Create A record
  community.general.ipwcli_dns:
    dnsname: example.com
    type: A
    container: ZoneOne
    address: 127.0.0.1

- name: Remove SRV record if exists
  community.general.ipwcli_dns:
    dnsname: _sip._tcp.test.example.com
    type: SRV
    container: ZoneOne
    ttl: 100
    state: absent
    target: example.com
    port: 5060

- name: Create NAPTR record
  community.general.ipwcli_dns:
    dnsname: test.example.com
    type: NAPTR
    preference: 10
    container: ZoneOne
    ttl: 100
    order: 10
    service: 'SIP+D2T'
    replacement: '_sip._tcp.test.example.com.'
    flags: S
'''

RETURN = '''
record:
    description: The created record from the input params
    type: str
    returned: always
'''

from ansible.module_utils.basic import AnsibleModule
import os


class ResourceRecord(object):

    def __init__(self, module):
        self.module = module
        self.dnsname = module.params['dnsname']
        self.dnstype = module.params['type']
        self.container = module.params['container']
        self.address = module.params['address']
        self.ttl = module.params['ttl']
        self.state = module.params['state']
        self.priority = module.params['priority']
        self.weight = module.params['weight']
        self.port = module.params['port']
        self.target = module.params['target']
        self.order = module.params['order']
        self.preference = module.params['preference']
        self.flags = module.params['flags']
        self.service = module.params['service']
        self.replacement = module.params['replacement']
        self.user = module.params['username']
        self.password = module.params['password']

    def create_naptrrecord(self):
        # create NAPTR record with the given params
        record = ('naptrrecord %s -set ttl=%s;container=%s;order=%s;preference=%s;flags="%s";service="%s";replacement="%s"'
                  % (self.dnsname, self.ttl, self.container, self.order, self.preference, self.flags, self.service, self.replacement))
        return record

    def create_srvrecord(self):
        # create SRV record with the given params
        record = ('srvrecord %s -set ttl=%s;container=%s;priority=%s;weight=%s;port=%s;target=%s'
                  % (self.dnsname, self.ttl, self.container, self.priority, self.weight, self.port, self.target))
        return record

    def create_arecord(self):
        # create A record with the given params
        if self.dnstype == 'AAAA':
            record = 'aaaarecord %s %s -set ttl=%s;container=%s' % (self.dnsname, self.address, self.ttl, self.container)
        else:
            record = 'arecord %s %s -set ttl=%s;container=%s' % (self.dnsname, self.address, self.ttl, self.container)

        return record

    def list_record(self, record):
        # check if the record exists via list on ipwcli
        search = 'list %s' % (record.replace(';', '&&').replace('set', 'where'))
        cmd = [self.module.get_bin_path('ipwcli', True)]
        cmd.append('-user=%s' % (self.user))
        cmd.append('-password=%s' % (self.password))
        rc, out, err = self.module.run_command(cmd, data=search)

        if 'Invalid username or password' in out:
            self.module.fail_json(msg='access denied at ipwcli login: Invalid username or password')

        if (('ARecord %s' % self.dnsname in out and rc == 0) or ('SRVRecord %s' % self.dnsname in out and rc == 0) or
                ('NAPTRRecord %s' % self.dnsname in out and rc == 0)):
            return True, rc, out, err

        return False, rc, out, err

    def deploy_record(self, record):
        # check what happens if create fails on ipworks
        stdin = 'create %s' % (record)
        cmd = [self.module.get_bin_path('ipwcli', True)]
        cmd.append('-user=%s' % (self.user))
        cmd.append('-password=%s' % (self.password))
        rc, out, err = self.module.run_command(cmd, data=stdin)

        if 'Invalid username or password' in out:
            self.module.fail_json(msg='access denied at ipwcli login: Invalid username or password')

        if '1 object(s) created.' in out:
            return rc, out, err
        else:
            self.module.fail_json(msg='record creation failed', stderr=out)

    def delete_record(self, record):
        # check what happens if create fails on ipworks
        stdin = 'delete %s' % (record.replace(';', '&&').replace('set', 'where'))
        cmd = [self.module.get_bin_path('ipwcli', True)]
        cmd.append('-user=%s' % (self.user))
        cmd.append('-password=%s' % (self.password))
        rc, out, err = self.module.run_command(cmd, data=stdin)

        if 'Invalid username or password' in out:
            self.module.fail_json(msg='access denied at ipwcli login: Invalid username or password')

        if '1 object(s) were updated.' in out:
            return rc, out, err
        else:
            self.module.fail_json(msg='record deletion failed', stderr=out)


def run_module():
    # define available arguments/parameters a user can pass to the module
    module_args = dict(
        dnsname=dict(type='str', required=True),
        type=dict(type='str', required=True, choices=['A', 'AAAA', 'SRV', 'NAPTR']),
        container=dict(type='str', required=True),
        address=dict(type='str', required=False),
        ttl=dict(type='int', required=False, default=3600),
        state=dict(type='str', default='present', choices=['absent', 'present']),
        priority=dict(type='int', required=False, default=10),
        weight=dict(type='int', required=False, default=10),
        port=dict(type='int', required=False),
        target=dict(type='str', required=False),
        order=dict(type='int', required=False),
        preference=dict(type='int', required=False),
        flags=dict(type='str', required=False, choices=['S', 'A', 'U', 'P']),
        service=dict(type='str', required=False),
        replacement=dict(type='str', required=False),
        username=dict(type='str', required=True),
        password=dict(type='str', required=True, no_log=True)
    )

    # define result
    result = dict(
        changed=False,
        stdout='',
        stderr='',
        rc=0,
        record=''
    )

    # supports check mode
    module = AnsibleModule(
        argument_spec=module_args,
        required_if=[
            ['type', 'A', ['address']],
            ['type', 'AAAA', ['address']],
            ['type', 'SRV', ['port', 'target']],
            ['type', 'NAPTR', ['preference', 'order', 'service', 'replacement']],
        ],
        supports_check_mode=True
    )

    user = ResourceRecord(module)

    if user.dnstype == 'NAPTR':
        record = user.create_naptrrecord()
    elif user.dnstype == 'SRV':
        record = user.create_srvrecord()
    elif user.dnstype == 'A' or user.dnstype == 'AAAA':
        record = user.create_arecord()

    found, rc, out, err = user.list_record(record)

    if found and user.state == 'absent':
        if module.check_mode:
            module.exit_json(changed=True)
        rc, out, err = user.delete_record(record)
        result['changed'] = True
        result['record'] = record
        result['rc'] = rc
        result['stdout'] = out
        result['stderr'] = err
    elif not found and user.state == 'present':
        if module.check_mode:
            module.exit_json(changed=True)
        rc, out, err = user.deploy_record(record)
        result['changed'] = True
        result['record'] = record
        result['rc'] = rc
        result['stdout'] = out
        result['stderr'] = err
    else:
        result['changed'] = False
        result['record'] = record
        result['rc'] = rc
        result['stdout'] = out
        result['stderr'] = err

    module.exit_json(**result)


def main():
    run_module()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = """
module: nxos_ntp
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Manages core NTP configuration.
description:
- Manages core NTP configuration.
version_added: 1.0.0
author:
- Jason Edelman (@jedelman8)
options:
  server:
    description:
    - Network address of NTP server.
    type: str
  peer:
    description:
    - Network address of NTP peer.
    type: str
  key_id:
    description:
    - Authentication key identifier to use with given NTP server or peer or keyword
      'default'.
    type: str
  prefer:
    description:
    - Makes given NTP server or peer the preferred NTP server or peer for the device.
    choices:
    - enabled
    - disabled
    type: str
  vrf_name:
    description:
    - Makes the device communicate with the given NTP server or peer over a specific
      VRF or keyword 'default'.
    type: str
  source_addr:
    description:
    - Local source address from which NTP messages are sent or keyword 'default'.
    type: str
  source_int:
    description:
    - Local source interface from which NTP messages are sent. Must be fully qualified
      interface name or keyword 'default'
    type: str
  state:
    description:
    - Manage the state of the resource.
    default: present
    choices:
    - present
    - absent
    type: str
"""

EXAMPLES = """
# Set NTP Server with parameters
- cisco.nxos.nxos_ntp:
    server: 1.2.3.4
    key_id: 32
    prefer: enabled
    host: '{{ inventory_hostname }}'
    username: '{{ un }}'
    password: '{{ pwd }}'
"""

RETURN = """
proposed:
    description: k/v pairs of parameters passed into module
    returned: always
    type: dict
    sample: {"address": "192.0.2.2", "key_id": "48",
            "peer_type": "server", "prefer": "enabled",
            "source": "192.0.2.3", "source_type": "source"}
existing:
    description:
        - k/v pairs of existing ntp server/peer
    returned: always
    type: dict
    sample: {"address": "192.0.2.2", "key_id": "32",
            "peer_type": "server", "prefer": "enabled",
            "source": "ethernet2/1", "source_type": "source-interface"}
end_state:
    description: k/v pairs of ntp info after module execution
    returned: always
    type: dict
    sample: {"address": "192.0.2.2", "key_id": "48",
            "peer_type": "server", "prefer": "enabled",
            "source": "192.0.2.3", "source_type": "source"}
updates:
    description: command sent to the device
    returned: always
    type: list
    sample: ["ntp server 192.0.2.2 prefer key 48",
             "no ntp source-interface ethernet2/1", "ntp source 192.0.2.3"]
changed:
    description: check to see if a change was made on the device
    returned: always
    type: bool
    sample: true
"""

import re

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    load_config,
    nxos_argument_spec,
    run_commands,
)


def execute_show_command(command, module, command_type="cli_show"):
    if "show run" not in command:
        output = "json"
    else:
        output = "text"

    commands = [{"command": command, "output": output}]
    return run_commands(module, commands)


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def get_ntp_source(module):
    source_type = None
    source = None
    command = "show run | inc ntp.source"
    output = execute_show_command(
        command, module, command_type="cli_show_ascii"
    )

    if output:
        try:
            if "interface" in output[0]:
                source_type = "source-interface"
            else:
                source_type = "source"
            source = output[0].split()[2].lower()
        except (AttributeError, IndexError):
            source_type = None
            source = None

    return source_type, source


def get_ntp_peer(module):
    command = "show run | inc ntp.(server|peer)"
    ntp_peer_list = []
    response = execute_show_command(
        command, module, command_type="cli_show_ascii"
    )

    if response:
        if isinstance(response, list):
            ntp = response[0]
        else:
            ntp = response
        if ntp:
            ntp_regex = (
                r".*ntp\s(server\s(?P<address>\S+)|peer\s(?P<peer_address>\S+))"
                r"\s*((?P<prefer>prefer)\s*)?(use-vrf\s(?P<vrf_name>\S+)\s*)?"
                r"(key\s(?P<key_id>\d+))?.*"
            )

            split_ntp = ntp.splitlines()
            for peer_line in split_ntp:
                if "access-group" in peer_line:
                    continue
                ntp_peer = {}
                try:
                    peer_address = None
                    vrf_name = "default"
                    prefer = None
                    key_id = None
                    match_ntp = re.match(ntp_regex, peer_line, re.DOTALL)
                    group_ntp = match_ntp.groupdict()

                    address = group_ntp["address"]
                    peer_address = group_ntp["peer_address"]
                    prefer = group_ntp["prefer"]
                    vrf_name = group_ntp["vrf_name"]
                    key_id = group_ntp["key_id"]

                    if prefer is not None:
                        prefer = "enabled"
                    else:
                        prefer = "disabled"

                    if address is not None:
                        peer_type = "server"
                    elif peer_address is not None:
                        peer_type = "peer"
                        address = peer_address

                    args = dict(
                        peer_type=peer_type,
                        address=address,
                        prefer=prefer,
                        vrf_name=vrf_name,
                        key_id=key_id,
                    )

                    ntp_peer = dict((k, v) for k, v in args.items())
                    ntp_peer_list.append(ntp_peer)
                except AttributeError:
                    ntp_peer_list = []

    return ntp_peer_list


def get_ntp_existing(address, peer_type, module):
    peer_dict = {}
    peer_server_list = []

    peer_list = get_ntp_peer(module)
    for peer in peer_list:
        if peer["address"] == address:
            peer_dict.update(peer)
        else:
            peer_server_list.append(peer)

    source_type, source = get_ntp_source(module)

    if source_type is not None and source is not None:
        peer_dict["source_type"] = source_type
        peer_dict["source"] = source

    return (peer_dict, peer_server_list)


def set_ntp_server_peer(peer_type, address, prefer, key_id, vrf_name):
    command_strings = []

    if prefer:
        command_strings.append(" prefer")
    if key_id:
        command_strings.append(" key {0}".format(key_id))
    if vrf_name:
        command_strings.append(" use-vrf {0}".format(vrf_name))

    command_strings.insert(0, "ntp {0} {1}".format(peer_type, address))

    command = "".join(command_strings)

    return command


def config_ntp(delta, existing):
    if (
        delta.get("address")
        or delta.get("peer_type")
        or delta.get("vrf_name")
        or delta.get("key_id")
        or delta.get("prefer")
    ):
        address = delta.get("address", existing.get("address"))
        peer_type = delta.get("peer_type", existing.get("peer_type"))
        key_id = delta.get("key_id", existing.get("key_id"))
        prefer = delta.get("prefer", existing.get("prefer"))
        vrf_name = delta.get("vrf_name", existing.get("vrf_name"))
        if delta.get("key_id") == "default":
            key_id = None
    else:
        peer_type = None
        prefer = None

    source_type = delta.get("source_type")
    source = delta.get("source")

    if prefer:
        if prefer == "enabled":
            prefer = True
        elif prefer == "disabled":
            prefer = False

    if source:
        source_type = delta.get("source_type", existing.get("source_type"))

    ntp_cmds = []
    if peer_type:
        if existing.get("peer_type") and existing.get("address"):
            ntp_cmds.append(
                "no ntp {0} {1}".format(
                    existing.get("peer_type"), existing.get("address")
                )
            )
        ntp_cmds.append(
            set_ntp_server_peer(peer_type, address, prefer, key_id, vrf_name)
        )
    if source:
        existing_source_type = existing.get("source_type")
        existing_source = existing.get("source")
        if existing_source_type and source_type != existing_source_type:
            ntp_cmds.append(
                "no ntp {0} {1}".format(existing_source_type, existing_source)
            )
        if source == "default":
            if existing_source_type and existing_source:
                ntp_cmds.append(
                    "no ntp {0} {1}".format(
                        existing_source_type, existing_source
                    )
                )
        else:
            ntp_cmds.append("ntp {0} {1}".format(source_type, source))

    return ntp_cmds


def main():
    argument_spec = dict(
        server=dict(type="str"),
        peer=dict(type="str"),
        key_id=dict(type="str"),
        prefer=dict(type="str", choices=["enabled", "disabled"]),
        vrf_name=dict(type="str"),
        source_addr=dict(type="str"),
        source_int=dict(type="str"),
        state=dict(choices=["absent", "present"], default="present"),
    )

    argument_spec.update(nxos_argument_spec)

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[["server", "peer"], ["source_addr", "source_int"]],
        supports_check_mode=True,
    )

    warnings = list()

    server = module.params["server"] or None
    peer = module.params["peer"] or None
    key_id = module.params["key_id"]
    prefer = module.params["prefer"]
    vrf_name = module.params["vrf_name"]
    source_addr = module.params["source_addr"]
    source_int = module.params["source_int"]
    state = module.params["state"]

    if source_int is not None:
        source_int = source_int.lower()

    if server:
        peer_type = "server"
        address = server
    elif peer:
        peer_type = "peer"
        address = peer
    else:
        peer_type = None
        address = None

    source_type = None
    source = None
    if source_addr:
        source_type = "source"
        source = source_addr
    elif source_int:
        source_type = "source-interface"
        source = source_int

    if key_id or vrf_name or prefer:
        if not server and not peer:
            module.fail_json(msg="Please supply the server or peer parameter")

    args = dict(
        peer_type=peer_type,
        address=address,
        key_id=key_id,
        prefer=prefer,
        vrf_name=vrf_name,
        source_type=source_type,
        source=source,
    )

    proposed = dict((k, v) for k, v in args.items() if v is not None)

    existing, peer_server_list = get_ntp_existing(address, peer_type, module)

    end_state = existing
    changed = False
    commands = []

    if state == "present":
        delta = dict(set(proposed.items()).difference(existing.items()))
        if delta.get("key_id") and delta.get("key_id") == "default":
            if not existing.get("key_id"):
                delta.pop("key_id")
        if delta:
            command = config_ntp(delta, existing)
            if command:
                commands.append(command)

    elif state == "absent":
        if existing.get("peer_type") and existing.get("address"):
            command = "no ntp {0} {1}".format(
                existing["peer_type"], existing["address"]
            )
            if command:
                commands.append([command])

        existing_source_type = existing.get("source_type")
        existing_source = existing.get("source")
        proposed_source_type = proposed.get("source_type")
        proposed_source = proposed.get("source")

        if proposed_source_type:
            if proposed_source_type == existing_source_type:
                if proposed_source == existing_source:
                    command = "no ntp {0} {1}".format(
                        existing_source_type, existing_source
                    )
                    if command:
                        commands.append([command])

    cmds = flatten_list(commands)
    if cmds:
        if module.check_mode:
            module.exit_json(changed=True, commands=cmds)
        else:
            changed = True
            load_config(module, cmds)
            end_state = get_ntp_existing(address, peer_type, module)[0]
            if "configure" in cmds:
                cmds.pop(0)

    results = {}
    results["proposed"] = proposed
    results["existing"] = existing
    results["updates"] = cmds
    results["changed"] = changed
    results["warnings"] = warnings
    results["end_state"] = end_state
    results["peer_server_list"] = peer_server_list

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# coding: utf-8 -*-

# (c) 2017, Wayne Witzel III <wayne@riotousliving.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: tower_job_list
author: "Wayne Witzel III (@wwitzel3)"
short_description: List Ansible Tower jobs.
description:
    - List Ansible Tower jobs. See
      U(https://www.ansible.com/tower) for an overview.
options:
    status:
      description:
        - Only list jobs with this status.
      choices: ['pending', 'waiting', 'running', 'error', 'failed', 'canceled', 'successful']
      type: str
    page:
      description:
        - Page number of the results to fetch.
      type: int
    all_pages:
      description:
        - Fetch all the pages and return a single result.
      type: bool
      default: 'no'
    query:
      description:
        - Query used to further filter the list of jobs. C({"foo":"bar"}) will be passed at C(?foo=bar)
      type: dict
extends_documentation_fragment: awx.awx.auth
'''


EXAMPLES = '''
- name: List running jobs for the testing.yml playbook
  tower_job_list:
    status: running
    query: {"playbook": "testing.yml"}
    tower_config_file: "~/tower_cli.cfg"
  register: testing_jobs
'''

RETURN = '''
count:
    description: Total count of objects return
    returned: success
    type: int
    sample: 51
next:
    description: next page available for the listing
    returned: success
    type: int
    sample: 3
previous:
    description: previous page available for the listing
    returned: success
    type: int
    sample: 1
results:
    description: a list of job objects represented as dictionaries
    returned: success
    type: list
    sample: [{"allow_simultaneous": false, "artifacts": {}, "ask_credential_on_launch": false,
              "ask_inventory_on_launch": false, "ask_job_type_on_launch": false, "failed": false,
              "finished": "2017-02-22T15:09:05.633942Z", "force_handlers": false, "forks": 0, "id": 2,
              "inventory": 1, "job_explanation": "", "job_tags": "", "job_template": 5, "job_type": "run"}, ...]
'''


from ..module_utils.tower_api import TowerAPIModule


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        status=dict(choices=['pending', 'waiting', 'running', 'error', 'failed', 'canceled', 'successful']),
        page=dict(type='int'),
        all_pages=dict(type='bool', default=False),
        query=dict(type='dict'),
    )

    # Create a module for ourselves
    module = TowerAPIModule(
        argument_spec=argument_spec,
        mutually_exclusive=[
            ('page', 'all_pages'),
        ]
    )

    # Extract our parameters
    query = module.params.get('query')
    status = module.params.get('status')
    page = module.params.get('page')
    all_pages = module.params.get('all_pages')

    job_search_data = {}
    if page:
        job_search_data['page'] = page
    if status:
        job_search_data['status'] = status
    if query:
        job_search_data.update(query)
    if all_pages:
        job_list = module.get_all_endpoint('jobs', **{'data': job_search_data})
    else:
        job_list = module.get_endpoint('jobs', **{'data': job_search_data})

    # Attempt to look up jobs based on the status
    module.exit_json(**job_list['json'])


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
---
module: faz_device
author: Luke Weighall (@lweighall)
short_description: Add or remove device
description:
  - Add or remove a device or list of devices to FortiAnalyzer Device Manager. ADOM Capable.

options:
  adom:
    description:
      - The ADOM the configuration should belong to.
    required: true
    default: root
    type: str

  mode:
    description:
      - Add or delete devices. Or promote unregistered devices that are in the FortiAnalyzer "waiting pool"
    required: false
    default: add
    choices: ["add", "delete", "promote"]
    type: str

  device_username:
    description:
      - The username of the device being added to FortiAnalyzer.
    required: false
    type: str

  device_password:
    description:
      - The password of the device being added to FortiAnalyzer.
    required: false
    type: str

  device_ip:
    description:
      - The IP of the device being added to FortiAnalyzer.
    required: false
    type: str

  device_unique_name:
    description:
      - The desired "friendly" name of the device being added to FortiAnalyzer.
    required: false
    type: str

  device_serial:
    description:
      - The serial number of the device being added to FortiAnalyzer.
    required: false
    type: str

  os_type:
    description:
      - The os type of the device being added (default 0).
    required: true
    choices: ["unknown", "fos", "fsw", "foc", "fml", "faz", "fwb", "fch", "fct", "log", "fmg", "fsa", "fdd", "fac"]
    type: str

  mgmt_mode:
    description:
      - Management Mode of the device you are adding.
    choices: ["unreg", "fmg", "faz", "fmgfaz"]
    required: true
    type: str

  os_minor_vers:
    description:
      - Minor OS rev of the device.
    required: true
    type: str

  os_ver:
    description:
      - Major OS rev of the device
    required: true
    choices: ["unknown", "0.0", "1.0", "2.0", "3.0", "4.0", "5.0", "6.0"]
    type: str

  platform_str:
    description:
      - Required for determine the platform for VM platforms. ie FortiGate-VM64
    required: false
    type: str

  faz_quota:
    description:
      - Specifies the quota for the device in FAZ
    required: False
    type: str
'''

EXAMPLES = '''
- name: DISCOVER AND ADD DEVICE A PHYSICAL FORTIGATE
  community.fortios.faz_device:
    adom: "root"
    device_username: "admin"
    device_password: "admin"
    device_ip: "10.10.24.201"
    device_unique_name: "FGT1"
    device_serial: "FGVM000000117994"
    state: "present"
    mgmt_mode: "faz"
    os_type: "fos"
    os_ver: "5.0"
    minor_rev: 6


- name: DISCOVER AND ADD DEVICE A VIRTUAL FORTIGATE
  community.fortios.faz_device:
    adom: "root"
    device_username: "admin"
    device_password: "admin"
    device_ip: "10.10.24.202"
    device_unique_name: "FGT2"
    mgmt_mode: "faz"
    os_type: "fos"
    os_ver: "5.0"
    minor_rev: 6
    state: "present"
    platform_str: "FortiGate-VM64"

- name: DELETE DEVICE FGT01
  community.fortios.faz_device:
    adom: "root"
    device_unique_name: "ansible-fgt01"
    mode: "delete"

- name: DELETE DEVICE FGT02
  community.fortios.faz_device:
    adom: "root"
    device_unique_name: "ansible-fgt02"
    mode: "delete"

- name: PROMOTE FGT01 IN FAZ BY IP
  community.fortios.faz_device:
    adom: "root"
    device_password: "fortinet"
    device_ip: "10.7.220.151"
    device_username: "ansible"
    mgmt_mode: "faz"
    mode: "promote"


- name: PROMOTE FGT02 IN FAZ
  community.fortios.faz_device:
    adom: "root"
    device_password: "fortinet"
    device_unique_name: "ansible-fgt02"
    device_username: "ansible"
    mgmt_mode: "faz"
    mode: "promote"

'''

RETURN = """
api_result:
  description: full API response, includes status code and message
  returned: always
  type: str
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.connection import Connection
from ansible_collections.community.fortios.plugins.module_utils.fortianalyzer.fortianalyzer import FortiAnalyzerHandler
from ansible_collections.community.fortios.plugins.module_utils.fortianalyzer.common import FAZBaseException
from ansible_collections.community.fortios.plugins.module_utils.fortianalyzer.common import FAZCommon
from ansible_collections.community.fortios.plugins.module_utils.fortianalyzer.common import FAZMethods
from ansible_collections.community.fortios.plugins.module_utils.fortianalyzer.common import DEFAULT_RESULT_OBJ
from ansible_collections.community.fortios.plugins.module_utils.fortianalyzer.common import FAIL_SOCKET_MSG


def faz_add_device(faz, paramgram):
    """
    This method is used to add devices to the faz or delete them
    """

    datagram = {
        "adom": paramgram["adom"],
        "device": {"adm_usr": paramgram["device_username"], "adm_pass": paramgram["device_password"],
                   "ip": paramgram["ip"], "name": paramgram["device_unique_name"],
                   "mgmt_mode": paramgram["mgmt_mode"], "os_type": paramgram["os_type"],
                   "mr": paramgram["os_minor_vers"]}
    }

    if paramgram["platform_str"] is not None:
        datagram["device"]["platform_str"] = paramgram["platform_str"]

    if paramgram["sn"] is not None:
        datagram["device"]["sn"] = paramgram["sn"]

    if paramgram["device_action"] is not None:
        datagram["device"]["device_action"] = paramgram["device_action"]

    if paramgram["faz.quota"] is not None:
        datagram["device"]["faz.quota"] = paramgram["faz.quota"]

    url = '/dvm/cmd/add/device/'
    response = faz.process_request(url, datagram, FAZMethods.EXEC)
    return response


def faz_delete_device(faz, paramgram):
    """
    This method deletes a device from the FAZ
    """
    datagram = {
        "adom": paramgram["adom"],
        "device": paramgram["device_unique_name"],
    }

    url = '/dvm/cmd/del/device/'
    response = faz.process_request(url, datagram, FAZMethods.EXEC)
    return response


def faz_get_unknown_devices(faz):
    """
    This method gets devices with an unknown management type field
    """

    faz_filter = ["mgmt_mode", "==", "0"]

    datagram = {
        "filter": faz_filter
    }

    url = "/dvmdb/device"
    response = faz.process_request(url, datagram, FAZMethods.GET)

    return response


def faz_approve_unregistered_device_by_ip(faz, paramgram):
    """
    This method approves unregistered devices by ip.
    """
    # TRY TO FIND DETAILS ON THIS UNREGISTERED DEVICE
    unknown_devices = faz_get_unknown_devices(faz)
    target_device = None
    if unknown_devices[0] == 0:
        for device in unknown_devices[1]:
            if device["ip"] == paramgram["ip"]:
                target_device = device
    else:
        return "No devices are waiting to be registered!"

    # now that we have the target device details...fill out the datagram and make the call to promote it
    if target_device is not None:
        target_device_paramgram = {
            "adom": paramgram["adom"],
            "ip": target_device["ip"],
            "device_username": paramgram["device_username"],
            "device_password": paramgram["device_password"],
            "device_unique_name": paramgram["device_unique_name"],
            "sn": target_device["sn"],
            "os_type": target_device["os_type"],
            "mgmt_mode": paramgram["mgmt_mode"],
            "os_minor_vers": target_device["mr"],
            "os_ver": target_device["os_ver"],
            "platform_str": target_device["platform_str"],
            "faz.quota": target_device["faz.quota"],
            "device_action": paramgram["device_action"]
        }

        add_device = faz_add_device(faz, target_device_paramgram)
        return add_device

    return str("Couldn't find the desired device with ip: " + str(paramgram["device_ip"]))


def faz_approve_unregistered_device_by_name(faz, paramgram):
    # TRY TO FIND DETAILS ON THIS UNREGISTERED DEVICE
    unknown_devices = faz_get_unknown_devices(faz)
    target_device = None
    if unknown_devices[0] == 0:
        for device in unknown_devices[1]:
            if device["name"] == paramgram["device_unique_name"]:
                target_device = device
    else:
        return "No devices are waiting to be registered!"

    # now that we have the target device details...fill out the datagram and make the call to promote it
    if target_device is not None:
        target_device_paramgram = {
            "adom": paramgram["adom"],
            "ip": target_device["ip"],
            "device_username": paramgram["device_username"],
            "device_password": paramgram["device_password"],
            "device_unique_name": paramgram["device_unique_name"],
            "sn": target_device["sn"],
            "os_type": target_device["os_type"],
            "mgmt_mode": paramgram["mgmt_mode"],
            "os_minor_vers": target_device["mr"],
            "os_ver": target_device["os_ver"],
            "platform_str": target_device["platform_str"],
            "faz.quota": target_device["faz.quota"],
            "device_action": paramgram["device_action"]
        }

        add_device = faz_add_device(faz, target_device_paramgram)
        return add_device

    return str("Couldn't find the desired device with name: " + str(paramgram["device_unique_name"]))


def main():
    argument_spec = dict(
        adom=dict(required=False, type="str", default="root"),
        mode=dict(choices=["add", "delete", "promote"], type="str", default="add"),

        device_ip=dict(required=False, type="str"),
        device_username=dict(required=False, type="str"),
        device_password=dict(required=False, type="str", no_log=True),
        device_unique_name=dict(required=False, type="str"),
        device_serial=dict(required=False, type="str"),

        os_type=dict(required=False, type="str", choices=["unknown", "fos", "fsw", "foc", "fml",
                                                          "faz", "fwb", "fch", "fct", "log", "fmg",
                                                          "fsa", "fdd", "fac"]),
        mgmt_mode=dict(required=False, type="str", choices=["unreg", "fmg", "faz", "fmgfaz"]),
        os_minor_vers=dict(required=False, type="str"),
        os_ver=dict(required=False, type="str", choices=["unknown", "0.0", "1.0", "2.0", "3.0", "4.0", "5.0", "6.0"]),
        platform_str=dict(required=False, type="str"),
        faz_quota=dict(required=False, type="str")
    )

    required_if = [
        ['mode', 'delete', ['device_unique_name']],
        ['mode', 'add', ['device_serial', 'device_username',
                         'device_password', 'device_unique_name', 'device_ip', 'mgmt_mode', 'platform_str']]

    ]

    module = AnsibleModule(argument_spec, supports_check_mode=True, required_if=required_if, )

    # START SESSION LOGIC
    paramgram = {
        "adom": module.params["adom"],
        "mode": module.params["mode"],
        "ip": module.params["device_ip"],
        "device_username": module.params["device_username"],
        "device_password": module.params["device_password"],
        "device_unique_name": module.params["device_unique_name"],
        "sn": module.params["device_serial"],
        "os_type": module.params["os_type"],
        "mgmt_mode": module.params["mgmt_mode"],
        "os_minor_vers": module.params["os_minor_vers"],
        "os_ver": module.params["os_ver"],
        "platform_str": module.params["platform_str"],
        "faz.quota": module.params["faz_quota"],
        "device_action": None
    }
    # INSERT THE PARAMGRAM INTO THE MODULE SO WHEN WE PASS IT TO MOD_UTILS.FortiManagerHandler IT HAS THAT INFO

    if paramgram["mode"] == "add":
        paramgram["device_action"] = "add_model"
    elif paramgram["mode"] == "promote":
        paramgram["device_action"] = "promote_unreg"
    module.paramgram = paramgram

    # TRY TO INIT THE CONNECTION SOCKET PATH AND FortiManagerHandler OBJECT AND TOOLS
    faz = None
    if module._socket_path:
        connection = Connection(module._socket_path)
        faz = FortiAnalyzerHandler(connection, module)
        faz.tools = FAZCommon()
    else:
        module.fail_json(**FAIL_SOCKET_MSG)

    # BEGIN MODULE-SPECIFIC LOGIC -- THINGS NEED TO HAPPEN DEPENDING ON THE ENDPOINT AND OPERATION
    results = DEFAULT_RESULT_OBJ

    try:
        if paramgram["mode"] == "add":
            results = faz_add_device(faz, paramgram)
    except BaseException as err:
        raise FAZBaseException(msg="An error occurred trying to add the device. Error: " + str(err))

    try:
        if paramgram["mode"] == "promote":
            if paramgram["ip"] is not None:
                results = faz_approve_unregistered_device_by_ip(faz, paramgram)
            elif paramgram["device_unique_name"] is not None:
                results = faz_approve_unregistered_device_by_name(faz, paramgram)
    except BaseException as err:
        raise FAZBaseException(msg="An error occurred trying to promote the device. Error: " + str(err))

    try:
        if paramgram["mode"] == "delete":
            results = faz_delete_device(faz, paramgram)
    except BaseException as err:
        raise FAZBaseException(msg="An error occurred trying to delete the device. Error: " + str(err))

    # PROCESS RESULTS
    try:
        faz.govern_response(module=module, results=results,
                            ansible_facts=faz.construct_ansible_facts(results, module.params, paramgram))
    except BaseException as err:
        raise FAZBaseException(msg="An error occurred with govern_response(). Error: " + str(err))

    # This should only be hit if faz.govern_response is missed or failed somehow. In fact. It should never be hit.
    # But it's here JIC.
    return module.exit_json(**results[1])


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2017 Zim Kalinowski, <zikalino@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: azure_rm_containerinstance_info
version_added: "0.1.2"
short_description: Get Azure Container Instance facts
description:
    - Get facts of Container Instance.

options:
    resource_group:
        description:
            - The name of the resource group.
        required: True
    name:
        description:
            - The name of the container instance.
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
  - name: Get specific Container Instance facts
    azure_rm_containerinstance_info:
      resource_group: myResourceGroup
      name: myContainer

  - name: List Container Instances in a specified resource group name
    azure_rm_containerinstance_info:
      resource_group: myResourceGroup
'''

RETURN = '''
container_groups:
    description: A list of Container Instance dictionaries.
    returned: always
    type: complex
    contains:
        id:
            description:
                - The resource id.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ContainerInstance/contain
                     erGroups/myContainer"
        resource_group:
            description:
                - Resource group where the container exists.
            returned: always
            type: str
            sample: testrg
        name:
            description:
                - The resource name.
            returned: always
            type: str
            sample: mycontainers
        location:
            description:
                - The resource location.
            returned: always
            type: str
            sample: westus
        os_type:
            description:
                - The OS type of containers.
            returned: always
            type: str
            sample: linux
        ip_address:
            description:
                - IP address of the container instance.
            returned: always
            type: str
            sample: 173.15.18.1
        dns_name_label:
            description:
                - The Dns name label for the IP.
            returned: always
            type: str
            sample: mydomain
        ports:
            description:
                - List of ports exposed by the container instance.
            returned: always
            type: list
            sample: [ 80, 81 ]
        containers:
            description:
                - The containers within the container group.
            returned: always
            type: complex
            sample: containers
            contains:
                name:
                    description:
                        - The name of the container instance.
                    returned: always
                    type: str
                    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ContainerInstance
                             /containerGroups/myContainer"
                image:
                    description:
                        - The container image name.
                    returned: always
                    type: str
                    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ContainerInstance
                             /containerGroups/myContainer"
                memory:
                    description:
                        - The required memory of the containers in GB.
                    returned: always
                    type: float
                    sample: 1.5
                cpu:
                    description:
                        - The required number of CPU cores of the containers.
                    returned: always
                    type: int
                    sample: 1
                ports:
                    description:
                        - List of ports exposed within the container group.
                    returned: always
                    type: list
                    sample: [ 80, 81 ]
                commands:
                    description:
                        - List of commands to execute within the container instance in exec form.
                    returned: always
                    type: list
                    sample: [ "pip install abc" ]
                environment_variables:
                    description:
                        - List of container environment variables.
                    type: complex
                    contains:
                        name:
                            description:
                                - Environment variable name.
                            type: str
                        value:
                            description:
                                - Environment variable value.
                            type: str
        tags:
            description: Tags assigned to the resource. Dictionary of string:string pairs.
            type: dict
            sample: { "tag1": "abc" }
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
from ansible.module_utils.common.dict_transformations import _camel_to_snake

try:
    from msrestazure.azure_exceptions import CloudError
    from msrestazure.azure_operation import AzureOperationPoller
    from azure.mgmt.containerinstance import ContainerInstanceManagementClient
    from msrest.serialization import Model
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMContainerInstanceInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str'
            ),
            tags=dict(
                type='list'
            )
        )
        # store the results of the module operation
        self.results = dict(
            changed=False,
        )
        self.resource_group = None
        self.name = None

        super(AzureRMContainerInstanceInfo, self).__init__(self.module_arg_spec, supports_tags=False)

    def exec_module(self, **kwargs):

        is_old_facts = self.module._name == 'azure_rm_containerinstance_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_containerinstance_facts' module has been renamed to 'azure_rm_containerinstance_info'", version=(2.9, ))

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if (self.name is not None):
            self.results['containerinstances'] = self.get()
        elif (self.resource_group is not None):
            self.results['containerinstances'] = self.list_by_resource_group()
        else:
            self.results['containerinstances'] = self.list_all()
        return self.results

    def get(self):
        response = None
        results = []
        try:
            response = self.containerinstance_client.container_groups.get(resource_group_name=self.resource_group,
                                                                          container_group_name=self.name)
            self.log("Response : {0}".format(response))
        except CloudError as e:
            self.log('Could not get facts for Container Instances.')

        if response is not None and self.has_tags(response.tags, self.tags):
            results.append(self.format_item(response))

        return results

    def list_by_resource_group(self):
        response = None
        results = []
        try:
            response = self.containerinstance_client.container_groups.list_by_resource_group(resource_group_name=self.resource_group)
            self.log("Response : {0}".format(response))
        except CloudError as e:
            self.fail('Could not list facts for Container Instances.')

        if response is not None:
            for item in response:
                if self.has_tags(item.tags, self.tags):
                    results.append(self.format_item(item))

        return results

    def list_all(self):
        response = None
        results = []
        try:
            response = self.containerinstance_client.container_groups.list()
            self.log("Response : {0}".format(response))
        except CloudError as e:
            self.fail('Could not list facts for Container Instances.')

        if response is not None:
            for item in response:
                if self.has_tags(item.tags, self.tags):
                    results.append(self.format_item(item))

        return results

    def format_item(self, item):
        d = item.as_dict()
        containers = d['containers']
        ports = d['ip_address']['ports'] if 'ip_address' in d else []
        resource_group = d['id'].split('resourceGroups/')[1].split('/')[0]

        for port_index in range(len(ports)):
            ports[port_index] = ports[port_index]['port']

        for container_index in range(len(containers)):
            old_container = containers[container_index]
            new_container = {
                'name': old_container['name'],
                'image': old_container['image'],
                'memory': old_container['resources']['requests']['memory_in_gb'],
                'cpu': old_container['resources']['requests']['cpu'],
                'ports': [],
                'commands': old_container.get('command'),
                'environment_variables': old_container.get('environment_variables')
            }
            for port_index in range(len(old_container['ports'])):
                new_container['ports'].append(old_container['ports'][port_index]['port'])
            containers[container_index] = new_container

        d = {
            'id': d['id'],
            'resource_group': resource_group,
            'name': d['name'],
            'os_type': d['os_type'],
            'dns_name_label': d['ip_address'].get('dns_name_label'),
            'ip_address': d['ip_address']['ip'] if 'ip_address' in d else '',
            'ports': ports,
            'location': d['location'],
            'containers': containers,
            'restart_policy': _camel_to_snake(d.get('restart_policy')) if d.get('restart_policy') else None,
            'tags': d.get('tags', None)
        }
        return d


def main():
    AzureRMContainerInstanceInfo()


if __name__ == '__main__':
    main()
